// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _GEOM_DATA_H_
#define _GEOM_DATA_H_

#include "utils/types.h"
#include "utils/index_oriented.h"
#include "utils/Index_on_set_function.h"

#include <vector>
#include <list>
#include <set>
//-----------------------------------------------------------------------------
struct Geom_data
{
    typedef std::vector< size_t >::iterator			Face_point_ind_iterator;
    typedef std::vector< size_t >::const_iterator		Face_point_ind_const_iterator;

    typedef std::vector< Index >::iterator				Cell_face_ind_iterator;
    typedef std::vector< Index >::const_iterator		Cell_face_ind_const_iterator;

    typedef std::vector< Point >::iterator				Point_iterator;
    typedef std::vector< Point >::const_iterator		Point_const_iterator;

    typedef std::pair<int, int> 						Index_pair;
    typedef std::vector<Index_pair> 					Index_pair_vector;

    std::vector< Point > points;

    //faces are vectors of point indices
    std::vector< std::vector< size_t > > face_points_ind;

    //cells are vectors of face oriented indices
    std::vector< std::vector< Index > > cell_faces_ind;

    //map face to both cells sharing the face
    Index_pair_vector face_to_cells_map;

    //walls are sets of faces indices. If face is not in the walls list it is assumed to be separate wall.
    std::list< std::set< size_t > > walls;

    //methods

    Geom_data() {}

    Geom_data ( std::ifstream & IN, double r = 0.0 )
    {
        read_geom_data ( IN, r );
    }

    ~Geom_data() {}

    void read_geom_data ( std::ifstream & IN, double r = 0.0 );
    void read_mesh_file ( std::ifstream & IN, Index_on_vec_function &funct );

    void statistics ( std::ostream &out = std::cout, bool detailed = false ) const;

    size_t numb_points() const
    {
        return points.size();
    };
    size_t numb_faces() const
    {
        return face_points_ind.size();
    };
    size_t numb_cells() const
    {
        return cell_faces_ind.size();
    };

    size_t numb_face_points ( size_t ind ) const
    {
        return face_points_ind[ind].size();
    };
    size_t numb_cell_faces ( size_t ind ) const
    {
        return cell_faces_ind[ind].size();
    };


    size_t face_index_back ( size_t i ) const
    {
        return face_points_ind[i].back();
    }
    size_t face_index_front ( size_t i ) const
    {
        return face_points_ind[i].front();
    }

    //iterators
    Face_point_ind_iterator face_point_ind_begin ( size_t ind )
    {
        return face_points_ind[ind].begin();
    }
    Face_point_ind_iterator face_point_ind_end ( size_t ind )
    {
        return face_points_ind[ind].end();
    }
    Face_point_ind_const_iterator face_point_ind_begin ( size_t ind ) const
    {
        return face_points_ind[ind].begin();
    }
    Face_point_ind_const_iterator face_point_ind_end ( size_t ind ) const
    {
        return face_points_ind[ind].end();
    }

    Cell_face_ind_iterator cell_face_ind_begin ( size_t ind )
    {
        return cell_faces_ind[ind].begin();
    }
    Cell_face_ind_iterator cell_face_ind_end ( size_t ind )
    {
        return cell_faces_ind[ind].end();
    }
    Cell_face_ind_const_iterator cell_face_ind_begin ( size_t ind ) const
    {
        return cell_faces_ind[ind].begin();
    }
    Cell_face_ind_const_iterator cell_face_ind_end ( size_t ind ) const
    {
        return cell_faces_ind[ind].end();
    }
};
#endif


