// Copyright (C) 2008 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _DIAGONALIZATION_H_
#define _DIAGONALIZATION_H_
extern "C"
{
#include "diagonalization/dsyevh3.h"
}
#include "diagonalization/eigen_sort.h"
#include "matrix/DenseMatrix.h"
#include "matrix/DenseVector.h"
//-----------------------------------------------------------------------------
//calculating eigenvalues and eigenvectors of the symmetric matrix given in e_vec, e_vec gets overwriten with eigenvectors in the columns
void inline diagonalize( DenseMatrix &e_vec, DenseVector &e_val, bool sign = true )
{
    //normal vectors Q[][i] normalized
    double A[3][3], Q[3][3], w[3];

    A[0][0] = e_vec.get(0,0);
    A[0][1] = e_vec.get(0,1);
    A[0][2] = e_vec.get(0,2);
    A[1][1] = e_vec.get(1,1);
    A[1][2] = e_vec.get(1,2);
    A[2][2] = e_vec.get(2,2);
    int result = dsyevh3( A, Q, w );
    if ( result )
        cell_error("error in diagonalization procedure.");

    for ( int i = 0; i < 3; ++i )
    {
        e_val(i) = w[i];
        for ( int j = 0; j < 3; ++j )
        {
            e_vec.set( i, j, Q[i][j] );
        }
    }
    if (sign)
        eigen_sort( e_val, e_vec );
    else
        eigen_sort_fabs( e_val, e_vec );
}
//-----------------------------------------------------------------------------
#endif
