// Copyright (C) 2008 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _DIAG_EIGEN_SORT_H_
#define _DIAG_EIGEN_SORT_H_
#include "matrix/DenseMatrix.h"
#include "matrix/DenseVector.h"

//-----------------------------------------------------------------------------
void inline eigen_sort(DenseVector &v, DenseMatrix &m)
{
	size_t n = v.size();
	assert( n == m.size(0) && n == m.size(1) );
	size_t k;
	double p;
	for( size_t i = 0; i < n-1; ++i )
	{
		p = v(k=i);
		for( size_t j  = i+1; j < n; ++j )
			if( v(j) >= p ) 
				p = v(k=j);
		if(k != i)
		{
			v(k) = v(i);
			v(i) = p;
			for( size_t j = 0; j < n; ++j )
			{
				p = m.get(j,i);
				m.set(j,i, m.get(j,k));
				m.set(j,k, p);
			}
		}
	}
}
//-----------------------------------------------------------------------------
void inline eigen_sort_fabs(DenseVector &v, DenseMatrix &m)
{
	size_t n = v.size();
	assert( n == m.size(0) && n == m.size(1) );
	size_t k;
	double p;
	for( size_t i = 0; i < n-1; ++i )
	{
		p = v(k=i);
		for( size_t j  = i+1; j < n; ++j )
			if( fabs(v(j)) >= fabs(p) ) 
				p = v(k=j);
		if(k != i)
		{
			v(k) = v(i);
			v(i) = p;
			for( size_t j = 0; j < n; ++j )
			{
				p = m.get(j,i);
				m.set(j,i, m.get(j,k));
				m.set(j,k, p);
			}
		}
	}
}
//-----------------------------------------------------------------------------
#endif
