// Copyright (C) 2004-2006 Anders Logg.
// Licensed under the GNU GPL Version 2.
//
// Modified by Pawel Krupinski 2007

#include "fem/Finite_element.h"
#include "fem/Bilinear_form.h"
#include "utils/error_macros.h"

//-----------------------------------------------------------------------------
Bilinear_form::Bilinear_form ( size_t num_functions )
    : Form ( num_functions ), _test ( 0 ), _trial ( 0 ), test_nodes ( 0 ), trial_nodes ( 0 )
{
    // Do nothing
}
//-----------------------------------------------------------------------------
Bilinear_form::~Bilinear_form()
{
    if ( test_nodes ) delete [] test_nodes;
    if ( trial_nodes ) delete [] trial_nodes;
}
//-----------------------------------------------------------------------------
Finite_element& Bilinear_form::test() const
{
    cell_assert ( _test ); // Should be created by child class
    return *_test;
}
//-----------------------------------------------------------------------------
Finite_element& Bilinear_form::trial() const
{
    cell_assert ( _trial ); // Should be created by child class
    return *_trial;
}
//-----------------------------------------------------------------------------
void Bilinear_form::update ( Affine_map_shell<Mesh_face> const& map ) const
{
    // Update coefficients
    update_coefficients ( map );
    // Initialize block
    const size_t m = _test->spacedim();
    const size_t n = _trial->spacedim();
    if ( !block )
        block = new double[m*n];
    for ( size_t i = 0; i < m*n; i++ )
        block[i] = 0.0;

    // Initialize nodes
    if ( !test_nodes )
        test_nodes = new size_t[_test->spacedim()];
    if ( !trial_nodes )
        trial_nodes = new size_t[_trial->spacedim()];
}
//-----------------------------------------------------------------------------
void Bilinear_form::update ( double val, double *par )
{
    //nothing to update by defualt
}
//-----------------------------------------------------------------------------
