// Copyright (C) 2004-2006 Anders Logg.
// Licensed under the GNU GPL Version 2.
//
// Modified by Pawel Krupinski 2007

#ifndef __BILINEAR_FORM_H
#define __BILINEAR_FORM_H

#include "Form.h"


/// Bilinear_form represents a bilinear form a(v, u) with arguments v
/// and u basis functions of the finite element space defined by a
/// pair of finite elements (test and trial).

class Bilinear_form : public Form
{
public:
    /// Constructor
    Bilinear_form ( size_t num_functions = 0 );
    /// Destructor
    virtual ~Bilinear_form();
    /// Compute element matrix (interior contribution)
    virtual void eval ( double block[], const Affine_map_shell<Mesh_face>& map ) const = 0;
    /// Compute element matrix (boundary contribution)
    virtual void eval_boundary ( double block[], const Affine_map_shell<Mesh_face>& map ) const = 0;
    /// Return finite element defining the test space
    Finite_element& test() const;
    /// update the parameters of the form (optional)
    virtual void update ( double val, double *par );
    /// Update map to current cell
    void update ( Affine_map_shell<Mesh_face> const& map ) const;
    /// Return finite element defining the trial space
    Finite_element& trial() const;
    /// Friends
    friend class FEM;

protected:
    // Finite element defining the test space
    Finite_element* _test;
    // Finite element defining the trial space
    Finite_element* _trial;
    // Local-to-global mapping for test space
    mutable size_t* test_nodes;
    // Local-to-global mapping for trial space
    mutable size_t* trial_nodes;
};

#endif
