// Copyright (C) 2006 Anders Logg.
// Licensed under the GNU GPL Version 2.
//
// Modified by Pawel Krupinski 2007

#include "fem/elements/invalid_element.h"
#include "fem/Finite_element.h"
#include "fem/ConstantFunction.h"

//-----------------------------------------------------------------------------
ConstantFunction::ConstantFunction(double value): Generic_function(), value(value), _mesh(0), mesh_local(false)
{
	// Do nothing
}
//-----------------------------------------------------------------------------
ConstantFunction::ConstantFunction(const ConstantFunction& f)
: Generic_function(),
value(f.value), _mesh(f._mesh), mesh_local(false)
{
	// Do nothing
}
//-----------------------------------------------------------------------------
ConstantFunction::~ConstantFunction()
{
	// Delete mesh if local
	if ( mesh_local )
		delete _mesh;
}
//-----------------------------------------------------------------------------
double ConstantFunction::operator()(const Point& p, size_t i)
{
	return value;
}
//-----------------------------------------------------------------------------
double ConstantFunction::operator() (const Mesh_node* vertex, size_t i)
{
	return value;
}
//-----------------------------------------------------------------------------
void ConstantFunction::sub(size_t i)
{
	// Do nothing (value same for all components anyway)
}
//-----------------------------------------------------------------------------
void ConstantFunction::interpolate(double coefficients[], Affine_map_shell<Mesh_face> const& map, Finite_element& element)
{
	// Evaluate function at interpolation points
	for (size_t i = 0; i < element.spacedim(); i++)
		coefficients[i] = value;
}
//-----------------------------------------------------------------------------
size_t ConstantFunction::vectordim() const
{
	cell_error("DenseVector dimension unknown for constant function.");
	return 0;
}
//-----------------------------------------------------------------------------
DenseVector& ConstantFunction::vector()
{
	cell_error("No vector associated with function (and none can be attached).");
	return *(new DenseVector()); // Code will not be reached, make compiler happy
}
//-----------------------------------------------------------------------------
Mesh& ConstantFunction::mesh()
{
	if ( !_mesh )
		cell_error("No mesh associated with function (try attaching one).");
	return *_mesh;
}
//-----------------------------------------------------------------------------
Finite_element& ConstantFunction::element()
{
	cell_error("No finite element associated with function (an none can be attached).");
	return *(Invalid_element::instance()); // Code will not be reached, make compiler happy
}
//-----------------------------------------------------------------------------
void ConstantFunction::attach(DenseVector& x, bool local)
{
	cell_error("Cannot attach vectors to constant functions.");
}
//-----------------------------------------------------------------------------
void ConstantFunction::attach(Mesh& mesh, bool local)
{
	// Delete old mesh if local
	if ( mesh_local )
		delete _mesh;

	// Attach new mesh
	_mesh = &mesh;
	mesh_local = local;
}
//-----------------------------------------------------------------------------
void ConstantFunction::attach(Finite_element const& element, bool local)
{
	cell_error("Cannot attach finite elements to constant functions.");
}
//-----------------------------------------------------------------------------
