// Copyright (C) 2006 Anders Logg.
// Licensed under the GNU GPL Version 2.
//
// Modified by Pawel Krupinski 2007

#ifndef __CONSTANT_FUNCTION_H
#define __CONSTANT_FUNCTION_H

#include "fem/Generic_function.h"

/// This class implements the functionality for a function
/// with a constant value.

class ConstantFunction : public Generic_function
{
public:

	/// Create function from given value
	ConstantFunction(double value);

	/// Copy constructor
	ConstantFunction(const ConstantFunction& f);

	/// Destructor
	~ConstantFunction();

	/// Evaluate function at given point
	double operator() (const Point& point, size_t i);

	/// Evaluate function at given vertex
	double operator() (const Mesh_node* vertex, size_t i);

	// Restrict to sub function or component (if possible)
	void sub(size_t i);

	/// Compute interpolation of function onto local finite element space
	void interpolate(double coefficients[], Affine_map_shell<Mesh_face> const& map, Finite_element& element);

	/// Return vector dimension of function
	size_t vectordim() const;

	/// Calling this function generates an error (no vector associated)
	DenseVector& vector();

	/// Return mesh associated with function (if any)
	Mesh& mesh();

	/// Calling this function generates an error (no element associated)
	Finite_element& element();

	/// Calling this function generates an error (no vector can be attached)
	void attach(DenseVector& x, bool local);

	/// Attach mesh to function
	void attach(Mesh& mesh, bool local);

	/// Calling this function generates an error (no element can be attached)
	void attach(Finite_element const& element, bool local);

private:

	// Value of constant function
	double value;

	// Pointer to mesh associated with function (null if none)
	Mesh* _mesh;

	// True if mesh is local (not a reference to another mesh)
	bool mesh_local;

};

#endif
