// Copyright (C) 2006 Anders Logg.
// Licensed under the GNU GPL Version 2.
//
// Modified by Pawel Krupinski 2007

#include <sstream>

#include "fem/Finite_element_spec.h"

//-----------------------------------------------------------------------------
Finite_element_spec::Finite_element_spec()
{
    init ( "unspecified", "unspecified", 0, 0 );
}
//-----------------------------------------------------------------------------
Finite_element_spec::Finite_element_spec ( std::string type )
{
    init ( "type", "unspecified", 0, 0 );
}
//-----------------------------------------------------------------------------
Finite_element_spec::Finite_element_spec ( std::string type, std::string shape,
        size_t degree, size_t vectordim )
{
    init ( type, shape, degree, vectordim );
}
//-----------------------------------------------------------------------------
void Finite_element_spec::init ( std::string type, std::string shape,
                                 size_t degree, size_t vectordim )
{
    _type = type;
    _shape = shape;
    _degree = degree;
    _vectordim = vectordim;
}
//-----------------------------------------------------------------------------
std::string Finite_element_spec::type() const
{
    return _type;
}
//-----------------------------------------------------------------------------
std::string Finite_element_spec::shape() const
{
    return _shape;
}
//-----------------------------------------------------------------------------
size_t Finite_element_spec::degree() const
{
    return _degree;
}
//-----------------------------------------------------------------------------
size_t Finite_element_spec::vectordim() const
{
    return _vectordim;
}
//-----------------------------------------------------------------------------
std::string Finite_element_spec::repr() const
{
    std::ostringstream stream;

    if ( _vectordim > 0 )
    {
        stream << "[ " << _type << " finite element of degree " << _degree
               << " on a " << _shape << " with " << _vectordim << " vector dimensions ]";
    }
    else
    {
        stream << "[ " << _type << " finite element of degree " << _degree
               << " on a " << _shape << " ]";
    }

    return stream.str();
}
//-----------------------------------------------------------------------------
std::ostream& operator<< ( std::ostream& stream, const Finite_element_spec& spec )
{
    stream << spec.repr();

    return stream;
}
//-----------------------------------------------------------------------------
