// Copyright (C) 2004-2005 Anders Logg.
// Licensed under the GNU GPL Version 2.
//
// Modified by Pawel Krupinski 2007

#include <iostream>
#include "Form.h"
#include "utils/error_macros.h"


//-----------------------------------------------------------------------------
Form::Form(size_t num_functions)
  : c(0), num_functions(num_functions), block(NULL)
{
  // Initialize list of functions
  if ( num_functions > 0 )
  {
    // Reserve list of functions
    functions.clear();
    functions.reserve(num_functions);

    // Reserve list of elements
    elements.clear();
    elements.reserve(num_functions);

    // Initialize coefficients
    c = new double* [num_functions];
    for (size_t i = 0; i < num_functions; i++)
      c[i] = 0;
  }
}
//-----------------------------------------------------------------------------
Form::~Form()
{
  // Delete coefficients
  if ( c )
  {
    for (size_t i = 0; i < num_functions; i++)
      delete [] c[i];
    delete [] c;
  }

  if( block )
	  delete[] block;
}
//-----------------------------------------------------------------------------
void Form::add(Function& f, Finite_element* element)
{
  if ( functions.size() == num_functions )
    cell_error("All functions already added.");
  
  // Get number of new function
  size_t i = functions.size();
  // Set finite element for function, but only for discrete functions
  if ( f.type() == Function::discrete )
    f.attach(*element);
  // Add function and element
  functions.push_back(&f);
  elements.push_back(element);
  // Initialize coefficients
  c[i] = new double[element->spacedim()];
  for (size_t j = 0; j < element->spacedim(); j++)
    c[i][j] = 0.0;
}
//-----------------------------------------------------------------------------
void Form::update_coefficients(Affine_map_shell<Mesh_face> const& map) const
{
  cell_assert(num_functions == functions.size());
  // Interpolate all functions to the current element
  for (size_t i = 0; i < num_functions; i++)
  {
    cell_assert(functions[i]);
    functions[i]->interpolate(c[i], map, *elements[i]);
  }
}
//-----------------------------------------------------------------------------
