// Copyright (C) 2004-2005 Anders Logg.
// Licensed under the GNU GPL Version 2.
//
// Modified by Pawel Krupinski 2007

#ifndef __FORM_H
#define __FORM_H

#include "fem/Function.h"
#include "fem/Affine_map_shell.h"
#include "fem/Finite_element.h"


class Form
{
public:

    /// Constructor
    Form ( size_t num_functions );

    /// Destructor
    virtual ~Form();

    /// Update map to current cell
    virtual void update ( Affine_map_shell<Mesh_face> const& map ) const = 0;

    /// Friends
    friend class FEM;

protected:

    // Add function
    void add ( Function &f, Finite_element* element );

    // Update coefficients
    void update_coefficients ( Affine_map_shell<Mesh_face> const& map ) const;

    // List of finite elements for functions (coefficients)
    std::vector<Finite_element*> elements;

    // List of functions (coefficients)
    std::vector<Function *> functions;

    // Coefficients of functions projected to current element
    mutable double** c;

    // Number of functions
    size_t num_functions;

    //block of values for assembly
    mutable double* block;

};

#endif
