// Copyright (C) 2005 Anders Logg.
// Licensed under the GNU GPL Version 2.
//
// Modified by Pawel Krupinski 2007

#ifndef __GENERIC_FUNCTION_H
#define __GENERIC_FUNCTION_H

#include "utils/types.h"
#include "fem/Local_function_data.h"
#include "matrix/DenseVector.h"

class Mesh ;
class Mesh_node;
class Mesh_face;
template <class ME> class Affine_map_shell;
class Finite_element;

/// This class serves as a base class/interface for implementations
/// of specific function representations.

class Generic_function
{
public:

    /// Constructor
    Generic_function();

    /// Destructor
    virtual ~Generic_function();

    /// Evaluate function at given point
    virtual double operator() (const Point& point, size_t i) = 0;

    /// Evaluate function at given vertex
    virtual double operator() (const Mesh_node* vertex, size_t i) = 0;

    // Restrict to sub function or component (if possible)
    virtual void sub(size_t i) = 0;

    /// Compute interpolation of function onto local finite element space
    virtual void interpolate(double coefficients[], Affine_map_shell<Mesh_face> const& map, Finite_element& element) = 0;

    /// Return vector dimension of function
    virtual size_t vectordim() const = 0;

    /// Return vector associated with function (if any)
    virtual DenseVector& vector() = 0;

    /// Return mesh associated with function (if any)
    virtual Mesh& mesh() = 0;

    /// Return element associated with function (if any)
    virtual Finite_element const& element() = 0;

    /// Attach vector to function
    virtual void attach(DenseVector& x, bool local) = 0;

    /// Attach mesh to function
    virtual void attach(Mesh& mesh, bool local) = 0;

    /// Attach finite element to function
    virtual void attach(Finite_element const& element, bool local) = 0;

    /// Print debug information
    virtual void info(bool detail) const {"No debug info specified for this function\n";
    }
protected:

    // Local storage for interpolation and evaluation
    Local_function_data local;

};

#endif
