// Copyright (C) 2004-2006 Anders Logg.
// Licensed under the GNU GPL Version 2.
//
// Modified by Pawel Krupinski 2007
#ifndef __LINEAR_FORM_H
#define __LINEAR_FORM_H

#include "Form.h"

/// Linear_form represents a linear form L(v) with argument v (the
/// test function) a basis function of the finite element space
/// defined by a finite element.

class Linear_form : public Form
{
public:
    /// Constructor
    Linear_form ( size_t num_functions = 0 );
    /// Destructor
    virtual ~Linear_form();
    /// Compute element vector (interior contribution)
    virtual void eval ( double block[], const Affine_map_shell<Mesh_face>& map ) const = 0;
    /// Compute element vector (boundary contribution)
    virtual void eval_boundary ( double block[], const Affine_map_shell<Mesh_face>& map ) const = 0;
    /// update the parameters of the form (optional)
    virtual void update ( double val, double *par );
    /// Update map to current cell
    void update ( Affine_map_shell<Mesh_face> const& map ) const;
    /// Return finite element defining the test space
    Finite_element& test();
    /// Friends
    friend class FEM;

protected:
    // Finite element defining the test space
    Finite_element* _test;
    // Local-to-global mapping for test space
    mutable size_t* test_nodes;
};

#endif
