// Copyright (C) 2011 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _LOAD_CASE_H_
#define _LOAD_CASE_H_

#include "fem/surface_load.h"
#include "fem/concentrated_load.h"
#include "fem/line_load.h"
//-----------------------------------------------------------------------------
class Load_case
{
public:
    typedef std::vector<Surface_load *>::iterator Surface_load_iterator;
    typedef std::vector<Surface_load *>::const_iterator Surface_load_const_iterator;
    typedef std::vector<Concentrated_load *>::iterator Concentrated_load_iterator;
    typedef std::vector<Concentrated_load *>::const_iterator Concentrated_load_const_iterator;
    typedef std::vector<Line_load *>::iterator Line_load_iterator;
    typedef std::vector<Line_load *>::const_iterator Line_load_const_iterator;

    Load_case ( Mesh_handle m ) : m_mesh(m), m_time(0.0)
    {
      
    }
    ~Load_case() 
    {
      
    }
    void add_load ( Surface_load *l )
    {
        m_surface_loads.push_back ( l );
    }
    void add_load ( Concentrated_load *l )
    {
        m_concentrated_loads.push_back ( l );
    }
    void add_load ( Line_load *l )
    {
        m_line_loads.push_back ( l );
    }

    Surface_load_iterator surface_loads_begin()
    {
        return m_surface_loads.begin();
    }
    Surface_load_iterator surface_loads_end()
    {
        return m_surface_loads.end();
    }
    Surface_load_const_iterator surface_loads_begin() const
    {
        return m_surface_loads.begin();
    }
    Surface_load_const_iterator surface_loads_end() const
    {
        return m_surface_loads.end();
    }

    Concentrated_load_iterator concentrated_loads_begin()
    {
        return m_concentrated_loads.begin();
    }
    Concentrated_load_iterator concentrated_loads_end()
    {
        return m_concentrated_loads.end();
    }
    Concentrated_load_const_iterator concentrated_loads_begin() const
    {
        return m_concentrated_loads.begin();
    }
    Concentrated_load_const_iterator concentrated_loads_end() const
    {
        return m_concentrated_loads.end();
    }

    Line_load_iterator line_loads_begin()
    {
        return m_line_loads.begin();
    }
    Line_load_iterator line_loads_end()
    {
        return m_line_loads.end();
    }
    Line_load_const_iterator line_loads_begin() const
    {
        return m_line_loads.begin();
    }
    Line_load_const_iterator line_loads_end() const
    {
        return m_line_loads.end();
    }

    double time() const
    {
        return m_time;
    }
    void update_time(double t) const
    {
      m_time = t;
    }

private:
    Mesh_handle m_mesh;
    std::vector<Surface_load *> m_surface_loads;
    std::vector<Concentrated_load *> m_concentrated_loads;
    std::vector<Line_load *> m_line_loads;
    mutable double m_time; //used in gradual application of load in Newton solver. Should be in range [0,1]
};
//-----------------------------------------------------------------------------
#endif
