// Copyright (C) 2005 Anders Logg.
// Licensed under the GNU GPL Version 2.
//
// Modified by Pawel Krupinski 2007

#include "fem/Finite_element.h"
#include "mesh/mesh_node.h"
#include "utils/types.h"
#include "fem/Local_function_data.h"

//-----------------------------------------------------------------------------
Local_function_data::Local_function_data() 
  : dofs(0), components(0), points(0), vertex_nodes(0), n(0)
{
  // Do nothing
}
//-----------------------------------------------------------------------------
Local_function_data::~Local_function_data()
{
  // Clear data if initialized
  clear();
}
//-----------------------------------------------------------------------------
void Local_function_data::init(const Finite_element& element)
{
  // Check if data is already initialized correctly
  if ( n == element.spacedim() )
    return;

  // Clear data if initialized
  clear();

  // Initialize local degrees of freedom
  dofs = new size_t[element.spacedim()];

  // Initialize local components
  components = new size_t[element.spacedim()];

  // Initialize local nodal points
  points = new Point[element.spacedim()];
  
  // Initialize local vertex nodes
  vertex_nodes = new size_t[element.tensordim()];

  // Save dimension of local function space
  n = element.spacedim();
}
//-----------------------------------------------------------------------------
void Local_function_data::clear()
{
  if ( dofs )
    delete [] dofs;
  dofs = 0;
  
  if ( components )
    delete [] components;
  components = 0;
  
  if ( points )
    delete [] points;
  points = 0;
  
  if ( vertex_nodes )
    delete [] vertex_nodes;
  vertex_nodes = 0;
}
//-----------------------------------------------------------------------------
