// Copyright (C) 2005 Anders Logg.
// Licensed under the GNU GPL Version 2.
//
// Modified by Pawel Krupinski 2007

#ifndef __LOCAL_FUNCTION_DATA_H
#define __LOCAL_FUNCTION_DATA_H

#include "utils/types.h"

class Finite_element;


/// Class collects containers for local storage used for interpolation
/// evaluation of functions on local elements.

class Local_function_data
{
public:

    /// Constructor
    Local_function_data();

    /// Destructor
    ~Local_function_data();

    /// Initialize data for given element
    void init(const Finite_element& element);

    /// Clear data
    void clear();

    /// Global numbers for local degrees of freedom
    size_t* dofs;

    /// Component indices for local degrees of freedom
    size_t* components;

    /// Interpolation points for local degrees of freedom
    Point* points;

    /// Vertex nodes for components of function
    size_t* vertex_nodes;

private:

    // Dimension of local function space
    size_t n;

};

#endif
