// Copyright (C) 2011 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _CONCENTRATED_LOAD_H_
#define _CONCENTRATED_LOAD_H_

#include "mesh/mesh_handle.h"
#include "fem/timed_load.h"
#include "utils/types.h"
#include <vector>
#include <iterator>
//-----------------------------------------------------------------------------
struct Node_load
{
    Node_load(Node_handle n, Vector l, double t) : node(n), load_factor(l), load_position(t) {}
    Node_handle node;
    Vector load_factor;
    double load_position;
};
//-----------------------------------------------------------------------------
class Concentrated_load : public Timed_load
{
public:
    typedef std::vector<Node_load>::iterator Load_iterator;
    typedef std::vector<Node_load>::const_iterator Load_const_iterator;

    //construct with default force and load position
    Concentrated_load( double f=1.0, double t=0.0):  m_load(f), m_load_pos(t) {}

    void add(Node_handle n , Vector l, double t=0.0)
    {
        m_nodes.push_back(Node_load(n,l,t));
    }

    template<class I>
    void add(I from, I beyond , Vector l, double t = 0.0)
    {
       base_add(typename std::iterator_traits<I>::iterator_category(), from, beyond , l, t);
    }


    double load() const {
        return m_load;
    }
    double load_position() const {
        return m_load_pos;
    }
    bool is_localized() const {
        return !m_nodes.empty();
    }

    void set_load_value(double l) {
      m_load = l;
    }
    Load_iterator loads_begin() {
        return m_nodes.begin();
    }
    Load_iterator loads_end() {
        return m_nodes.end();
    }
    Load_const_iterator loads_begin() const {
        return m_nodes.begin();
    }
    Load_const_iterator loads_end() const {
        return m_nodes.end();
    }

private:

    template<class I>
    void base_add(std::bidirectional_iterator_tag, I from, I beyond , Vector l, double t)
    {
        while (from != beyond)
        {
            m_nodes.push_back(Node_load(*from,l,t));
            ++from;
        }
    }

    template<class I>
    void base_add(std::random_access_iterator_tag, I from, I beyond , Vector l, double t)
    {
        m_nodes.reserve(beyond-from);
        while (from != beyond)
        {
            m_nodes.push_back(Node_load(*from,l,t));
            ++from;
        }
    }

    double m_load;
    double m_load_pos;
    std::vector<Node_load> m_nodes;
};
//-----------------------------------------------------------------------------
#endif
