#ifndef _HEX8P_RULE_H
#define _HEX8P_RULE_H

#include "utils/types.h"
//#include "fem/Affine_map.h"
#include "fem/quadrature_rule.h"


class Hex8P_rule: public Quadrature_rule
{
public:

	/// Constructor
	Hex8P_rule(): Quadrature_rule(){};

	/// Destructor
	~Hex8P_rule(){};

	/// Return number of integration nodes
	size_t spacedim() const { return 8; }

	/// Return number of the underlying shape dim
	size_t shapedim() const { return 3; }

	/// Return integration points 
	void points( Point p[] ) const
	{
		double temp = 1.0/sqrt(3.0);
		p[0] = Point(-temp, -temp, -temp);
		p[1] = Point( temp, -temp, -temp);
		p[2] = Point( temp,  temp, -temp);
		p[3] = Point( -temp, temp, -temp);
		p[4] = Point(-temp, -temp,  temp);
		p[5] = Point( temp, -temp,  temp);
		p[6] = Point( temp,  temp,  temp);
		p[7] = Point( -temp, temp,  temp);
	}

	/// Return integration weigths 
	void weights( double w[] ) const
	{
		w[0] = 1.0;
		w[1] = 1.0;
		w[2] = 1.0;
		w[3] = 1.0;
		w[4] = 1.0;
		w[5] = 1.0;
		w[6] = 1.0;
		w[7] = 1.0;
	}

	/// Return specification for the rule
	inline std::string type() const
	{
		return std::string("[ 8 point rule on a hexadedron ]");
	}

};


#endif
