// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
// Modified by Pawel Krupinski 2007
// Licensed under the GNU GPL Version 2.

#ifndef _HEX_LAG_SCAL_H_
#define _HEX_LAG_SCAL_H_

#include "mesh/mesh.h"
#include "utils/types.h"
#include "fem/Finite_element.h"
#include "utils/error_macros.h"

class Hex_Lag_scal : public Finite_element
{
public:
//-----------------------------------------------------------------------------
    ~Hex_Lag_scal()
    {
    }
//-----------------------------------------------------------------------------
    inline size_t spacedim() const
    {
        return 8;
    }
//-----------------------------------------------------------------------------
    inline size_t shapedim() const
    {
        return 3;
    }
//-----------------------------------------------------------------------------
    inline size_t tensordim() const
    {
        return 1;
    }
//-----------------------------------------------------------------------------
    ///Local shape function. Input: element node index, array of dofs
    double function ( int i, double *x ) const
    {
        Point const& pt = reference_points[i];
        return 0.125* ( 1.+pt.x() *x[0] ) * ( 1.+pt.y() *x[1] ) * ( 1.+pt.z() *x[2] );
    }
//-----------------------------------------------------------------------------
    ///Local derivative of shape function. Input: element node index, array of dofs, differentiation coordinate
    double derivative ( int i, double *x, Coordinate_type c_t ) const
    {
        Point const& pt = reference_points[i];
        switch ( c_t )
        {
        case X:
            return 0.125*pt.x() * ( 1.+pt.y() *x[1] ) * ( 1.+pt.z() *x[2] );
            break;
        case Y:
            return 0.125*pt.y() * ( 1.+pt.x() *x[0] ) * ( 1.+pt.z() *x[2] );
            break;
        case Z:
            return 0.125*pt.z() * ( 1.+pt.y() *x[1] ) * ( 1.+pt.x() *x[0] );
            break;
        default:
            cell_error ( "Invalid coordinate type." );
            return 0.0;
        }
    }
    //-----------------------------------------------------------------------------
    void points ( Point p[] ) const
    {
        p[0] = Point ( -1,-1,-1 );
        p[1] = Point ( 1,-1,-1 );
        p[2] = Point ( 1, 1,-1 );
        p[3] = Point ( -1, 1,-1 );
        p[4] = Point ( -1,-1, 1 );
        p[5] = Point ( 1,-1, 1 );
        p[6] = Point ( 1, 1, 1 );
        p[7] = Point ( -1, 1, 1 );
    }
//-----------------------------------------------------------------------------
    Finite_element_spec spec() const
    {
        Finite_element_spec s ( "Lagrange", "hexahedron", 1 );
        return s;
    }
//-----------------------------------------------------------------------------
    static Hex_Lag_scal *instance()
    {
        if ( !m_instance )
            m_instance = new Hex_Lag_scal;
        return m_instance;
    }
//-----------------------------------------------------------------------------
private:
    Hex_Lag_scal() : Finite_element(), reference_points ( spacedim() /tensordim() )
    {
        points ( &reference_points[0] );
    }
    std::vector<Point>	reference_points;
    static Hex_Lag_scal * m_instance;
};
#endif
