#ifndef _QUAD12P_RULE_H
#define _QUAD12P_RULE_H

#include "utils/types.h"
//#include "fem/Affine_map.h"
#include "fem/quadrature_rule.h"

//quadrature rule for a quad shell with 3 thickness integration points
class Quad12P_rule: public Quadrature_rule
{
public:

    /// Return number of integration nodes
    size_t spacedim() const
    {
        return 12;
    }

    /// Return number of the underlying shape dim
    size_t shapedim() const
    {
        return 2;
    }

    /// Return integration points
    void points ( Point p[] ) const
    {
        const double a = 1.0 / sqrt ( 3.0 );
        const double b = sqrt ( 3.0/5.0 );

        p[0] = Point ( -a, -a, -b );
        p[1] = Point ( a, -a, -b );
        p[2] = Point ( a,  a, -b );
        p[3] = Point ( -a, a, -b );

        p[4] = Point ( -a, -a, 0 );
        p[5] = Point ( a, -a, 0 );
        p[6] = Point ( a,  a, 0 );
        p[7] = Point ( -a, a, 0 );

        p[8] = Point ( -a, -a, b );
        p[9] = Point ( a, -a, b );
        p[10] = Point ( a,  a, b );
        p[11] = Point ( -a, a, b );

//                      p[11] = Point(-a, -a, -b);
//                      p[10] = Point( a, -a, -b);
//                      p[9] = Point( a,  a, -b);
//                      p[8] = Point( -a, a, -b);
//
//                      p[7] = Point(-a, -a, 0);
//                      p[6] = Point( a, -a, 0);
//                      p[5] = Point( a,  a, 0);
//                      p[4] = Point( -a, a, 0);
//
//                      p[3] = Point(-a, -a, b);
//                      p[2] = Point( a, -a, b);
//                      p[1] = Point( a,  a, b);
//                      p[0] = Point( -a, a, b);
    }

    /// Return integration weigths
    void weights ( double w[] ) const
    {
        const double wg = 5.0 / 9.0;
        const double wg1 = 8.0 / 9.0;
        w[0] = wg;
        w[1] = wg;
        w[2] = wg;
        w[3] = wg;

        w[4] = wg1;
        w[5] = wg1;
        w[6] = wg1;
        w[7] = wg1;

//         w[4] = wg;
//         w[5] = wg;
//         w[6] = wg;
//         w[7] = wg;

        w[8] = wg;
        w[9] = wg;
        w[10] = wg;
        w[11] = wg;
    }

    /// Return specification for the rule
    inline std::string type() const
    {
        return std::string ( "[ 12 point rule on a quad shell ]" );
    }

};


#endif
