// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.

#ifndef _QUAD_SHELL_H_
#define _QUAD_SHELL_H_

#include "mesh/mesh.h"
#include "utils/types.h"
#include "fem/Finite_element.h"
#include "utils/error_macros.h"



class Quad_shell : public Finite_element
{
public:
//-----------------------------------------------------------------------------
    ~Quad_shell()
    {
    }
//-----------------------------------------------------------------------------
    inline size_t spacedim() const
    {
        return 24;
    }
//-----------------------------------------------------------------------------
    inline size_t shapedim() const
    {
        return 3;
    }
//-----------------------------------------------------------------------------
    inline size_t tensordim() const
    {
        return 6;
    }
    //-----------------------------------------------------------------------------
    ///Local shape function. Input: element node index, array of dofs
    double function ( int i, double *x ) const
    {
        Point_2D const& pt = reference_points[i];
        return 0.25* ( 1.+pt.x() *x[0] ) * ( 1.+pt.y() *x[1] );
    }
    //-----------------------------------------------------------------------------
    ///Local derivative of shape function. Input: element node index, array of dofs, differentiation coordinate
    double derivative ( int i, double *x, Coordinate_type c_t ) const
    {
        Point_2D const& pt = reference_points[i];
        switch ( c_t )
        {
        case X:
            return 0.25*pt.x() * ( 1.+pt.y() *x[1] );
            break;
        case Y:
            return 0.25*pt.y() * ( 1.+pt.x() *x[0] );
            break;
        default:
            cell_error ( "Invalid coordinate type." );
            return 0.0;
        }
    }
    //-----------------------------------------------------------------------------
    void points ( Point_2D p[] ) const
    {
        p[0] = Point_2D ( -1,-1 );
        p[1] = Point_2D ( 1,-1 );
        p[2] = Point_2D ( 1, 1 );
        p[3] = Point_2D ( -1, 1 );
    }
//-----------------------------------------------------------------------------
    Finite_element_spec spec() const
    {
        Finite_element_spec s ( "Shell", "quadrilateral", 1 );
        return s;
    }
//-----------------------------------------------------------------------------
    static Quad_shell *instance()
    {
        if ( !m_instance )
            m_instance = new Quad_shell;
        return m_instance;
    }
//-----------------------------------------------------------------------------
private:
    Quad_shell() : Finite_element()
    {
        points ( reference_points );
    }

    Point_2D reference_points[4];
    static Quad_shell * m_instance;
};
#endif
