#ifndef _QUAD4P_RULE_H
#define _QUAD4P_RULE_H

#include "utils/types.h"
//#include "fem/Affine_map.h"
#include "fem/quadrature_rule.h"


class Triang9P_rule: public Quadrature_rule
{
public:

    /// Return number of integration nodes
    size_t spacedim() const
    {
        return 9;
    }

    /// Return number of the underlying shape dim
    size_t shapedim() const
    {
        return 2;
    }

    /// Return integration points
    void points ( Point p[] ) const
    {
        const double a = 1.0 / 6.0;
        const double b = 2.0 / 3.0;
        const double c = sqrt ( 3.0/5.0 );
        //point coordinates are r,s
        p[0] = Point_2D ( a, a, -c );
        p[1] = Point_2D ( b, a, -c );
        p[2] = Point_2D ( a, b, -c );

        p[3] = Point_2D ( a, a, 0 );
        p[4] = Point_2D ( b, a, 0 );
        p[5] = Point_2D ( a, b, 0 );

        p[6] = Point ( a, a, c );
        p[7] = Point ( b, a, c );
        p[8] = Point ( a, b, c );
    }
    /// Return integration weigths
    void wieghts ( double w[] ) const
    {
        const double a = 1.0 / 6.0;
        const double w = 5.0 / 9.0 * a;
        w[0] = w;
        w[1] = w;
        w[2] = w;
        w[3] = w;
        w[4] = w;
        w[5] = w;
        w[6] = w;
        w[7] = w;
        w[8] = w;
        w[9] = w;
    }

    /// Return specification for the rule
    inline std::string type() const
    {
        return std::string ( "[ 9 point rule on a triangle shell ]" );
    }

};
#endif
