// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.

#ifndef __HEXAHEDRAL_SHAPE_FUNCTION_H
#define __HEXAHEDRAL_SHAPE_FUNCTION_H

#include "utils/error_macros.h"
#include "fem/shape_function.h"
#include "utils/types.h"
//-----------------------------------------------------------------------------
//shape functions in local coordinate system description
class Hex_shape_function: public Shape_function
{
public:
    Hex_shape_function() : Shape_function() {}

    ~Hex_shape_function() {}

    ///Local shape function. Input: reference coordinates xa...,
    double function ( double xa, double x, double ya, double y, double za, double z ) const
    {
        return 0.125* ( 1.+xa*x ) * ( 1.+ya*y ) * ( 1.+za*z );
    }
    //-----------------------------------------------------------------------------
    double function ( Point pa, Point p ) const
    {
        return function ( pa.x(), p.x(), pa.y(), p.y(), pa.z(), p.z() );
    }
    //-----------------------------------------------------------------------------
    ///Local derivative of shape function. Input: reference coordinates xa...,
    double derivative ( double xa, double x, double ya, double y, double za, double z, Coordinate_type c_t ) const
    {
        switch ( c_t )
        {
        case X:
            return 0.125*xa* ( 1.+ya*y ) * ( 1.+za*z );
            break;
        case Y:
            return 0.125*ya* ( 1.+xa*x ) * ( 1.+za*z );
            break;
        case Z:
            return 0.125*za* ( 1.+ya*y ) * ( 1.+xa*x );
            break;
        default:
            cell_error ( "Invalid coordinate type." );
            return 0.0;
        }
    }
    //-----------------------------------------------------------------------------
    double derivative ( Point pa, Point p, Coordinate_type c_t ) const
    {
        return derivative ( pa.x(), p.x(), pa.y(), p.y(), pa.z(), p.z(), c_t );
    }
};
//-----------------------------------------------------------------------------
#endif
