// Modified by Pawel Krupinski 2007
// Licensed under the GNU GPL Version 2.

#ifndef _INVALID_ELEMENT_H_
#define _INVALID_ELEMENT_H_

// #include "mesh/mesh.h"
#include "utils/types.h"
#include "fem/Finite_element.h"

#include "utils/error_macros.h"



class Invalid_element : public Finite_element
{
public:
//-----------------------------------------------------------------------------
    ~Invalid_element()
    {
    }
//-----------------------------------------------------------------------------
    size_t spacedim() const
    {
        return INVALID_SIZE;
    }
//-----------------------------------------------------------------------------
    size_t shapedim() const
    {
        return INVALID_SIZE;
    }
    //-----------------------------------------------------------------------------
    size_t tensordim() const
    {
        return INVALID_SIZE;
    }
    //-----------------------------------------------------------------------------
    double function(int i, double *x) const
    {
        cell_error("Accessing data of incalid finite element");
        return 0.0;
    }
    //-----------------------------------------------------------------------------
    ///Local derivative of shape function. Input: element node index, array of dofs, differentiation coordinate
    double derivative(int i, double *x, Coordinate_type c_t) const
    {
        cell_error("Accessing data of incalid finite element");
        return 0.0;
    }
    //-----------------------------------------------------------------------------
    void points( Point p[] ) const
    {
       cell_error("Accessing data of incalid finite element");
    }
//-----------------------------------------------------------------------------
    Finite_element_spec spec() const
    {
        Finite_element_spec s("Invalid", "invalid", 0, 0);
        return s;
    }
//-----------------------------------------------------------------------------
    static Invalid_element *instance()
    {
        if (!m_instance)
            m_instance = new Invalid_element;
        return m_instance;
    }
//-----------------------------------------------------------------------------
private:
    Invalid_element() : Finite_element()
    {
    }
    static Invalid_element * m_instance;
};
#endif
