// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _FEM_UTILS_H_
#define _FEM_UTILS_H_

#include "utils/types.h"
#include <vector>

template<class ME> class Affine_map_shell;
class Vector3;
class Dense_matrix;
class Dense_vector;
//-----------------------------------------------------------------------------
//reduces tangent matrix in lamina coordinates to account for zero normal stress condition
void reduce_tangent_matrix(DenseMatrix &d);
//-----------------------------------------------------------------------------
//calculate norm of the vector taking into account only translational degrees of freedom (neglecting rotations)
double norm( DenseVector &vec, int n_nodes );
//-----------------------------------------------------------------------------
//Transform symmetric tensor to a Voigt vector
void to_voigt_vector(Matrix3s const& m, DenseVector &v);
//-----------------------------------------------------------------------------
//Transform Voigt vector to a symmetric tensor
void from_voigt_vector(DenseVector const&v, Matrix3s &m);
//-----------------------------------------------------------------------------
void rotate_B_matrix( DenseMatrix &res,  DenseMatrix const& b, rot_mat const& rot, bool init = true );
//-----------------------------------------------------------------------------
//rotates strain displacement matrix so it acts from global to local coordinates
void inv_rotate_B_matrix( DenseMatrix &res,  DenseMatrix const& b, rot_mat const& rot, bool init = true);
//-----------------------------------------------------------------------------
//rotate data (in approporiate form acceptable by rot_mat) from lamina to global for each integration point of current element in affine map. t == true means inverse rotation
template<class D>
void rotate_element_data(std::vector<D> & loc, std::vector<D> const& glob, Affine_map_shell<Mesh_face> const& map, bool t = false)
{
    int nint = map.q_rule().spacedim();
    if ( t )
        for (int i = 0; i < nint; ++i)
            loc[i] = map.coord_rot(i).inv_rotate(glob[i]);
    else
        for (int i = 0; i < nint; ++i)
            loc[i] = map.coord_rot(i).rotate(glob[i]);
}
//-----------------------------------------------------------------------------
///stiffen rotational degrees of freedom assuming 6 dofs per node and that rotational dofs come after the translational dofs
void adjust_shell_lumped_mass_mat(DenseVector &lum_m, Affine_map_shell<Mesh_face> const& map, double a);
//-----------------------------------------------------------------------------
#endif
