// Copyright (C) 2011 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#include "fem/elements/Hex1P_rule.h"
#include "fem/elements/Hex8P_rule.h"
#include "fem/elements/Quad4P_rule.h"
#include "fem/elements/Quad12P_rule.h"
#include "fem/quadrature_rule_base.h"
//-----------------------------------------------------------------------------
/// Factory function for creating new 3D rule
template<>
Quadrature_rule_base<Point>* Quadrature_rule_base<Point>::make_QRule ( std::string type )
{
    if ( type == "[ 1 point rule on a hexadedron ]" )
    {
        cell_info ( "Creating quadrature rule: %s.", type.c_str() );
        return new Hex1P_rule();
    }
    else if ( type == "[ 8 point rule on a hexadedron ]" )
    {
        cell_info ( "Creating quadrature rule: %s.", type.c_str() );
        return new Hex8P_rule();
    }
    else if ( type == "[ 12 point rule on a quad shell ]" )
    {
        cell_info ( "Creating quadrature rule: %s.", type.c_str() );
        return new Quad12P_rule();
    }

    cell_warning1 ( "%s", type.c_str() );
    cell_warning ( "Unable to create specified quadrature rule, no matching precompiled rule type found." );

    return 0;
}
//-----------------------------------------------------------------------------
/// Factory function for creating new 2D rule
template<>
Quadrature_rule_base<Point_2D>* Quadrature_rule_base<Point_2D>::make_QRule ( std::string type )
{
    if ( type == "[ 4 point rule on a quad ]" )
    {
        cell_info ( "Creating quadrature rule: %s.", type.c_str() );
        return new Quad4P_rule();
    }

    cell_warning1 ( "%s", type.c_str() );
    cell_warning ( "Unable to create specified quadrature rule, no matching precompiled rule type found." );

    return 0;
}
//-----------------------------------------------------------------------------
