// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _QUADRATURE_RULE_BASE_H_
#define _QUADRATURE_RULE_BASE_H_

#include <string>
#include "utils/error_macros.h"
//-----------------------------------------------------------------------------
//quadrature rule templated with points (2D or 3D)
template<class P>
class Quadrature_rule_base
{
public:
    typedef P Point_type;
    /// Constructor
    Quadrature_rule_base() {};
    /// Destructor
    virtual ~Quadrature_rule_base() {};
    /// Return number of integration nodes
    virtual size_t spacedim() const = 0;
    /// Return dimension of the underlying shape
    virtual size_t shapedim() const = 0;
    /// Return integration weigths
    virtual void weights ( double w[] ) const = 0;
    /// Return integration points
    virtual void points ( P p[] ) const = 0;
    /// Return integration dofs array
// 	virtual void dofs( double d[] ) const = 0;
    /// Return specification for element
    virtual std::string type() const = 0;
    /// Factory function for creating new rule
    static Quadrature_rule_base<P> * make_QRule ( std::string type );
    /// Display rule data
    void disp() const;
};
//-----------------------------------------------------------------------------
/// Display rule data
template<class P>
void Quadrature_rule_base<P>::disp() const
{
    cell_info ( "Quadrature rule data:" );
    cell_info ( type().c_str() );
    cell_info ( "  space dimension = %d", spacedim() );
    cell_info ( "  shape dimension = %d", shapedim() );
}
//-----------------------------------------------------------------------------
#endif
