// Copyright (C) 2011 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _SURFACE_LOAD_H_
#define _SURFACE_LOAD_H_

#include "mesh/mesh_handle.h"
#include "fem/timed_load.h"
#include <vector>
#include <iterator>
//-----------------------------------------------------------------------------
struct Face_load
{
    Face_load(Face_handle f, double l, double t) : face(f), load_factor(l), load_position(t) {}
    Face_handle face;
    double load_factor;
    double load_position;
};
//-----------------------------------------------------------------------------
class Surface_load : public Timed_load
{
public:
    typedef std::vector<Face_load>::iterator Load_iterator;
    typedef std::vector<Face_load>::const_iterator Load_const_iterator;

    //construct with default pressure and load position t in the director direction reference space (t should be in range -1:1)
    Surface_load( double p=0.0, double t=0.0): m_press(p), m_load_pos(t) {}

    void add(Face_handle f , double l = 1.0, double t = 0.0)
    {
        m_faces.push_back(Face_load(f,l,t));
    }

    template<class I>
    void add(I from, I beyond , double l = 1.0, double t = 0.0)
    {
        base_add(typename std::iterator_traits<I>::iterator_category(), from, beyond , l, t);
    }


    double load() const {
        return m_press;
    }
    double load_position() const {
        return m_load_pos;
    }
    bool is_localized() const {
        return !m_faces.empty();
    }

    Load_iterator loads_begin() {
        return m_faces.begin();
    }
    Load_iterator loads_end() {
        return m_faces.end();
    }
    Load_const_iterator loads_begin() const {
        return m_faces.begin();
    }
    Load_const_iterator loads_end() const {
        return m_faces.end();
    }

private:

    template<class I>
    void base_add(std::bidirectional_iterator_tag, I from, I beyond , double l, double t)
    {
        while (from != beyond)
        {
            m_faces.push_back(Face_load(*from,l,t));
            ++from;
        }
    }

    template<class I>
    void base_add(std::random_access_iterator_tag, I from, I beyond , double l, double t)
    {
        m_faces.reserve(beyond-from);
        while (from != beyond)
        {
            m_faces.push_back(Face_load(*from,l,t));
            ++from;
        }
    }

    double m_press;
    double m_load_pos;
    std::vector<Face_load> m_faces;
};
//-----------------------------------------------------------------------------
#endif
