// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef CELL_IO_VTK_OSTREAM_H
#define CELL_IO_VTK_OSTREAM_H

// #include <CGAL/basic.h>
#include "io/ofstream_base.h"
#include <iostream>
#include <assert.h>
//-----------------------------------------------------------------------------
class VTK_ostream
{
public:
    VTK_ostream()           : m_os ( 0 )  {}
    VTK_ostream ( std::ostream& o ) : m_os ( &o )
    {
        header();
    }
    ~VTK_ostream()
    {
        close();
    }
    void open ( std::ostream& o )
    {
        m_os = &o;
        header();
    }
    void close()
    {
        if ( m_os )
            footer();
        m_os = 0;
    }

    typedef const void* Const_void_ptr;

    operator Const_void_ptr () const
    {
        if ( m_os )
            return *m_os;
        return 0;
    }

    std::ostream& os()
    {
        assert ( m_os );
        return *m_os;
    }

    VTK_ostream& seekp ( std::streampos pos )
    {
        m_os->seekp ( pos );
        return *this;
    }
    std::streampos tellp ( )
    {
        return m_os->tellp();
    }
    std::streamsize width ( std::streamsize wide )
    {
        return m_os->width ( wide );
    }
protected:
    void header()
    {

        os() << "<VTKFile type=\"UnstructuredGrid\" version=\"0.1\">\n"
             <<	"<UnstructuredGrid>\n";
    }
    void footer()
    {
        os() << "</UnstructuredGrid>\n"
             <<"</VTKFile>\n";
    }

    std::ostream*  m_os;
};
//-----------------------------------------------------------------------------
inline VTK_ostream& operator<< ( VTK_ostream& os, const char* s )
{
    os.os() << s;
    return os;
}
//-----------------------------------------------------------------------------
inline VTK_ostream& operator<< ( VTK_ostream& os, double d )
{
    os.os() << d;
    return os;
}
//-----------------------------------------------------------------------------
inline VTK_ostream& operator<< ( VTK_ostream& os, int i )
{
    os.os() << i;
    return os;
}
//-----------------------------------------------------------------------------
typedef OfstreamBase<VTK_ostream> VTK_ofstream;
//-----------------------------------------------------------------------------
#endif
