// Copyright (C) 2011 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.

#include "material/anisotropic_material.h"
#include "fem/Affine_map_shell.h"
#include "matrix/Tensor4.h"

//-----------------------------------------------------------------------------
void Anisotropic_material::elasticity( DenseMatrix &etens, Affine_map_shell<Mesh_face> const& map, int i ) const
{
    double Ji = 1.0 / map.def_grad_det(i);
    Matrix3 const& F = map.def_grad(i);
    double lam = Ji*lambda_t();
    double m  = Ji*mu_t();
    Matrix3s b;
    map.left_cauchy_green(b,i);
    Vector3 a0 = map.cell().fem_data().fiber;
    Vector3 a = F*a0;
    
    double dlam = (lambda_l() - lambda_t())*Ji;
    double dm = (mu_l() - mu_t())*Ji;
    Matrix3s AxA = dyad(a);
    Tensor4s c = dyad1s(b)*lam + dyad4s(b)*(2.0*m); //isotropic part
    Tensor4s ca =  dyad1s(b, AxA)*(0.5*dlam) + dyad4s(b, AxA)*dm; //anisotropic part Behruz
//     Tensor4s ca =  dyad1s(b, AxA)*dlam + dyad4s(b, AxA)*(2.0*dm) - dyad1s(AxA)*(dlam+2.0*dm); //anisotropic part Delingette
    Tensor4s cc = c + ca;   
    cc.extract(etens);
    
}
//-----------------------------------------------------------------------------
void Anisotropic_material::stress( Matrix3s &s, Affine_map_shell<Mesh_face> const& map, int i ) const
{
    double Ji = 1.0 / map.def_grad_det(i);
    Matrix3 const& F = map.def_grad(i);
    double lam = Ji*lambda_t();
    double m  = Ji*mu_t();

    Matrix3s b;
    map.left_cauchy_green(b,i);
    Matrix3s C;
    map.right_cauchy_green(C,i);
    double trE = 0.5*(b.tr()-3.0);
    Matrix3s b2 = b*b;

    Vector3 a0 = map.cell().fem_data().fiber;

    Vector3 a = F*a0;
    double aEa = 0.5*(a0*(C*a0)-1.0);

    double dlam = (lambda_l() - lambda_t())*Ji;
    double dm = (mu_l() - mu_t())*Ji;

    Matrix3s AxA = dyad(a);
    // calculate anisotropic contribution to stress
    s = b*(lam*trE - m) + b2*m //isotropic part
      + b * (0.5 * dlam * aEa) + AxA * ( 0.5 * dlam * trE) + (b * AxA + AxA * b) * 0.5 * dm - AxA * dm; //anisotropic part Behruz
//       + b * (dlam * aEa) + AxA * (dlam * trE) + (b * AxA + AxA * b) * 2.0 * dm - AxA *(aEa*(dlam + 2.0 * dm)); //anisotropic part Delingette
}
 //-----------------------------------------------------------------------------
 