// Copyright (C) 2011 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _ANISOTROPIC_MATERIAL_H_
#define _ANISOTROPIC_MATERIAL_H_
#include "material/generic_material.h"
#include "matrix/Tensor4.h"
#include <utils/string_utils.h>
//Simple anisotropic material based on Venant-Kirchoff material plus anisotropic part
//-----------------------------------------------------------------------------
class Anisotropic_material: public Generic_material
{
public:
    ///Constructor takes Young moduluds in transverse, longitudonal direction and Poisson coefficient
    Anisotropic_material( double Et, double El, double nu ): m_E_young_long(El), m_E_young_trans(Et), m_nu_poisson(nu)
    {
        init();
        std::cout << "E_trans = " << m_E_young_trans << "\nE_long = " << m_E_young_long << "\nlam_trans = " << m_lambda_t << "\nlam_long = " << m_lambda_l <<  "\nmu_trans = " << m_mu_t << "\nmu_long = " << m_mu_l << "\n";
    }
    void update( double Et, double El, double nu )
    {
        m_E_young_trans = Et;
        m_E_young_long = El;
        m_nu_poisson = nu;
        init();
    }

    void elasticity( DenseMatrix &etens, Affine_map_shell<Mesh_face> const& map, int i ) const;
    void stress( Matrix3s &s, Affine_map_shell<Mesh_face> const& map, int i ) const;

    double lambda_l() const {
        return m_lambda_l;
    }
    double mu_l() const {
        return m_mu_l;
    }
    double lambda_t() const {
        return m_lambda_t;
    }
    double mu_t() const {
        return m_mu_t;
    }

    double El() const {
        return m_E_young_long;
    }
    double Et() const {
        return m_E_young_trans;
    }
    double nu() const {
        return m_nu_poisson;
    }
    std::string info() const 
    {
        return "Anisotropic material: Young modulus longitudinal = " + to_string(El()) + "Young modulus transverse = " + to_string(Et()) +"; Poisson ratio = " + to_string(nu()) + "\n";
    }
private:
    //Young modulus and Poisson ratio
    double m_E_young_trans; // in transverse to direction to anisotropy vector
    double m_E_young_long; // in longitudinal to anisotropy vector
    double m_nu_poisson;

    double m_lambda_l;
    double m_lambda_t;
    double m_mu_l;
    double m_mu_t;

    void init()
    {
        m_lambda_l = (m_nu_poisson * m_E_young_long /(1. + m_nu_poisson)/(1. - 2.*m_nu_poisson ));
        m_mu_l = (m_E_young_long * 0.5 / (1. + m_nu_poisson));
        m_lambda_t = (m_nu_poisson * m_E_young_trans /(1. + m_nu_poisson)/(1. - 2.*m_nu_poisson ));
        m_mu_t = (m_E_young_trans * 0.5 / (1. + m_nu_poisson));
    }
};
//-----------------------------------------------------------------------------
#endif
