// Copyright (C) 2012 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _FIBER_MANAGER_H_
#define _FIBER_MANAGER_H_

#include "mesh/mesh_handle.h"
#include "cell/cell_handle.h"
class State_interpolator;
///class for controling behavior of material fibers during simulation and their alignment depending on different conditions
//-----------------------------------------------------------------------------
class Fiber_manager
{
public:
    virtual void align_material_fibers(Mesh_handle &m) = 0;
};
//-----------------------------------------------------------------------------
///class for aligning fiber direction parallel to maximal principal stress
class Align_with_stress : public Fiber_manager
{
public:
    Align_with_stress(State_interpolator const& si) : m_state_int(si) {}
    void align_material_fibers(Mesh_handle &m);
    
private:
  State_interpolator const& m_state_int;
};
//-----------------------------------------------------------------------------
///class for aligning fiber direction parallel to maximal principal stress based on the diference between value of max principal and projected stress
class Align_with_stress_projection_treshold: public Fiber_manager
{
public:
    Align_with_stress_projection_treshold(State_interpolator const& si, double t) : m_state_int(si), m_treshold(t) {}
    void align_material_fibers(Mesh_handle &m);
    
private:
  State_interpolator const& m_state_int;
  double m_treshold;
};
//-----------------------------------------------------------------------------
///class for aligning fiber direction parallel to maximal principal stress averaged over compartments
class Align_with_stress_over_compartments : public Fiber_manager
{
public:
    Align_with_stress_over_compartments(State_interpolator& si, cell::Tissue_handle t) : m_state_int(si), m_tissue(t) {}
    void align_material_fibers(Mesh_handle &m);
    
private:
  State_interpolator& m_state_int;
  cell::Tissue_handle m_tissue;
};
//-----------------------------------------------------------------------------
///class for aligning fiber direction perpendicular to maximal principal strain
class Align_with_strain : public Fiber_manager
{
public:
    Align_with_strain(State_interpolator const& si) : m_state_int(si) {}
    void align_material_fibers(Mesh_handle &m);
    
private:
  State_interpolator const& m_state_int;
};
//-----------------------------------------------------------------------------
///class for aligning fiber direction parallel to maximal principal strain averaged over compartments
class Align_with_strain_over_compartments : public Fiber_manager
{
public:
    Align_with_strain_over_compartments(State_interpolator& si, cell::Tissue_handle t) : m_state_int(si), m_tissue(t) {}
    void align_material_fibers(Mesh_handle &m);
    
private:
  State_interpolator& m_state_int;
  cell::Tissue_handle m_tissue;
};
//-----------------------------------------------------------------------------
///class for aligning fiber direction parallel to maximal principal stress averaged over neighbor compartments
class Align_with_stress_averaged_over_neighbor_compartments : public Fiber_manager
{
public:
    Align_with_stress_averaged_over_neighbor_compartments(State_interpolator& si, cell::Tissue_handle t, double a) : m_state_int(si), m_tissue(t), m_alpha(a) {}
    void align_material_fibers(Mesh_handle &m);
    
private:
  State_interpolator& m_state_int;
  cell::Tissue_handle m_tissue;
  double m_alpha;
};
//-----------------------------------------------------------------------------
#endif