// Copyright (C) 2011 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.

#include "material/isotropic_material.h"
#include "fem/Affine_map_shell.h"
#include "matrix/Tensor4.h"
#include "fem/fem_utils.h"
//-----------------------------------------------------------------------------
void Isotropic_material::elasticity ( DenseMatrix &etens, Affine_map_shell<Mesh_face> const& map, int i ) const

{
    double Ji = 1.0 / map.def_grad_det ( i );

    double lam = Ji*lambda();
    double m  = Ji*mu();

    Matrix3s b;
    map.left_cauchy_green ( b,i );
    Tensor4s c = dyad1s ( b ) *lam + dyad4s ( b ) * ( 2.0*m );
    c.extract ( etens );
}
//-----------------------------------------------------------------------------
void Isotropic_material::stress ( Matrix3s &s, Affine_map_shell<Mesh_face> const& map, int i ) const
{
    double Ji = 1.0 / map.def_grad_det ( i );
    double lam = Ji*lambda();
    double m  = Ji*mu();

    Matrix3s b;
    map.left_cauchy_green ( b,i );

    double trE = 0.5* ( b.tr()-3 );
    Matrix3s b2 = b*b;
    s = b* ( lam*trE - m ) + b2*m;
}
//-----------------------------------------------------------------------------
