// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _ISOTROPIC_MATERIAL_H
#define _ISOTROPIC_MATERIAL_H

#include "material/generic_material.h"
#include <utils/string_utils.h>

//-----------------------------------------------------------------------------
class Isotropic_material: public Generic_material
{
public:

    ///constructors
    Isotropic_material( double E, double nu ): m_E_young(E), m_nu_poisson(nu)
    {
        init();
    }

    void update( double E, double nu )
    {
        m_E_young = E;
        m_nu_poisson = nu;
        init();
    }

    void elasticity( DenseMatrix &etens, Affine_map_shell<Mesh_face> const& map, int i ) const;
    void stress( Matrix3s &s, Affine_map_shell<Mesh_face> const& map, int i ) const;

    double lambda() const 
    {
        return m_lambda;
    }
    double mu() const 
    {
        return m_mu;
    }

    double E() const 
    {
        return m_E_young;
    }
    double nu() const 
    {
        return m_nu_poisson;
    }

    std::string info() const 
    {
        return "Isotropic material: Young modulus = " + to_string(E()) + "; Poisson ratio = " + to_string(nu()) + "\n";
    }
private:

    //Young modulus and Poisson ratio
    double m_E_young;
    double m_nu_poisson;

    double m_lambda;
    double m_mu;
    void init()
    {
        m_lambda = (m_nu_poisson * m_E_young /(1. + m_nu_poisson)/(1. - 2.*m_nu_poisson ));
        m_mu = (m_E_young * 0.5 / (1. + m_nu_poisson));
    }

};
//-----------------------------------------------------------------------------
#endif
