// Copyright (C) 2011 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _VEC_3_D_
#define _VEC_3_D_
//-----------------------------------------------------------------------------
#include <math.h>

class Vector3
{
public:
    // constructors
    Vector3() : x ( 0 ), y ( 0 ), z ( 0 ) {}
    Vector3 ( double X, double Y, double Z ) : x ( X ), y ( Y ), z ( Z ) {}

    // operators
    Vector3 operator + ( const Vector3& r ) const
    {
        return Vector3 ( x+r.x, y+r.y, z+r.z );
    }
    Vector3 operator - ( const Vector3& r ) const
    {
        return Vector3 ( x-r.x, y-r.y, z-r.z );
    }

    Vector3 operator * ( double a ) const
    {
        return Vector3 ( x*a, y*a, z*a );
    }
    Vector3 operator / ( double a ) const
    {
        return Vector3 ( x/a, y/a, z/a );
    }

    Vector3& operator += ( const Vector3& r )
    {
        x += r.x;
        y += r.y;
        z += r.z;
        return ( *this );
    }
    Vector3& operator -= ( const Vector3& r )
    {
        x -= r.x;
        y -= r.y;
        z -= r.z;
        return ( *this );
    }

    Vector3& operator *= ( double a )
    {
        x*=a;
        y*=a;
        z*=a;
        return ( *this );
    }
    Vector3& operator /= ( double a )
    {
        x/=a;
        y/=a;
        z/=a;
        return ( *this );
    }

    Vector3 operator - () const
    {
        return Vector3 ( -x, -y, -z );
    }

    // dot product
    double operator * ( const Vector3& r ) const
    {
        return ( x*r.x + y*r.y + z*r.z );
    }

    // cross product
    Vector3 operator ^ ( const Vector3& r ) const
    {
        return Vector3 ( y*r.z-z*r.y,z*r.x-x*r.z,x*r.y-y*r.x );
    }

    // normalize the vector
    double unit()
    {
        double d = sqrt ( x*x+y*y+z*z );
        if ( d != 0 )
        {
            double id=1.0/d;
            x*=id;
            y*=id;
            z*=id;
        }
        return d;
    }

    double norm() const
    {
        return sqrt ( x*x+y*y+z*z );
    }
public:
    double x, y, z;
};
//-----------------------------------------------------------------------------
#endif
