// Copyright (C) 2011 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#include "matrix/uBlasVector.h"
#include "matrix/principal_directions.h"
#include "utils/error_macros.h"
extern "C"
{
#include "diagonalization/dsyevh3.h"
}
//-----------------------------------------------------------------------------
principal_directions::principal_directions ( Matrix3s const& m )
{
    calculate_principal_directions ( m );
}
//-----------------------------------------------------------------------------
principal_directions::principal_directions ( DenseVector const& m )
{
    calculate_principal_directions ( m );
}
//-----------------------------------------------------------------------------
void principal_directions::calculate_principal_directions ( Matrix3s const& m, bool sign )
{
    //normal vectors Q[][i] normalized
    double A[3][3], Q[3][3], w[3];

    A[0][0] = m.xx();
    A[0][1] = m.xy();
    A[0][2] = m.xz();
    A[1][1] = m.yy();
    A[1][2] = m.yz();
    A[2][2] = m.zz();

    int result = dsyevh3 ( A, Q, w );

    if ( result )
        cell_error ( "error in diagonalization procedure." );

    for ( int i = 0; i < 3; ++i )
    {
        eigenvalues.diag ( i ) = w[i];
        for ( int j = 0; j < 3; ++j )
        {
            eigenvectors ( i, j ) = Q[i][j];
        }
    }
    if ( sign )
        eigen_sort();
    else
        eigen_sort_fabs();
}
//-----------------------------------------------------------------------------
void principal_directions::calculate_principal_directions ( DenseVector const& m, bool sign )
{
    //normal vectors Q[][i] normalized
    double A[3][3], Q[3][3], w[3];

    A[0][0] = m.get ( 0 );
    A[0][1] = m.get ( 3 );
    A[0][2] = m.get ( 5 );
    A[1][1] = m.get ( 1 );
    A[1][2] = m.get ( 4 );
    A[2][2] = m.get ( 2 );

    int result = dsyevh3 ( A, Q, w );

    if ( result )
        cell_error ( "error in diagonalization procedure." );

    for ( int i = 0; i < 3; ++i )
    {
        eigenvalues.diag ( i ) = w[i];
        for ( int j = 0; j < 3; ++j )
        {
            eigenvectors ( i, j ) = Q[i][j];
        }
    }
    if ( sign )
        eigen_sort();
    else
        eigen_sort_fabs();
}
//-----------------------------------------------------------------------------
void principal_directions::eigen_sort_fabs()
{
    size_t k;
    double p;
    for ( size_t i = 0; i < 2; ++i )
    {
        p = eigenvalues.diag ( k=i );
        for ( size_t j  = i+1; j < 3; ++j )
            if ( fabs ( eigenvalues.diag ( j ) ) >= fabs ( p ) )
                p = eigenvalues.diag ( k=j );
        if ( k != i )
        {
            eigenvalues.diag ( k ) = eigenvalues.diag ( i );
            eigenvalues.diag ( i ) = p;
            for ( size_t j = 0; j < 3; ++j )
            {
                p = eigenvectors ( j, i );
                eigenvectors ( j, i ) = eigenvectors ( j, k );
                eigenvectors ( j, k ) = p;
            }
        }
    }
}
//-----------------------------------------------------------------------------
void principal_directions::eigen_sort()
{
    size_t k;
    double p;
    for ( size_t i = 0; i < 2; ++i )
    {
        p = eigenvalues.diag ( k=i );
        for ( size_t j  = i+1; j < 3; ++j )
            if ( eigenvalues.diag ( j ) >= p )
                p = eigenvalues.diag ( k=j );
        if ( k != i )
        {
            eigenvalues.diag ( k ) = eigenvalues.diag ( i );
            eigenvalues.diag ( i ) = p;
            for ( size_t j = 0; j < 3; ++j )
            {

                p = eigenvectors ( j, i );
                eigenvectors ( j, i ) = eigenvectors ( j, k );
                eigenvectors ( j, k ) = p;
            }
        }
    }
}
//-----------------------------------------------------------------------------
