// Copyright (C) 2011 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _PRINCIPAL_DIRECTIONS_H_
#define _PRINCIPAL_DIRECTIONS_H_

#include "matrix/Matrix3.h"
#include "matrix/Vector3.h"
#include "matrix/DenseVector.h"
#include "utils/types.h"
#include <math.h>
//-----------------------------------------------------------------------------
struct principal_directions
{

  principal_directions(){}
  principal_directions(Matrix3s const& m);
  principal_directions(DenseVector const& m);

  ///calculate principal directions of a symmetric tensor
  void calculate_principal_directions(Matrix3s const& m, bool sign = true);
  ///calculate principal directions of a symmetric tensor in Voigt vector form
  void calculate_principal_directions(DenseVector const& m, bool sign = true);
  ///give i-th principal vector
  Vector3 vec(int i) const
  {
    return Vector3( eigenvectors(0,i), eigenvectors(1,i), eigenvectors(2,i) );
  }
  ///principal vector vectors in columns of the matrix
  Matrix3 const& vec() const
  {
    return eigenvectors;
  }
  ///give i-th principal vector
  Vector Vec(int i) const
  {
    return Vector( eigenvectors(0,i), eigenvectors(1,i), eigenvectors(2,i) );
  }
  ///give i-th principal value
  double val(int i) const
  {
    return eigenvalues.diag(i);
  }

  void eigen_sort();
  void eigen_sort_fabs();

  Matrix3d eigenvalues;
  Matrix3 eigenvectors;
};
//-----------------------------------------------------------------------------
#endif
