// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.

#include "mesh/boundary_builder.h"
#include "mesh/mesh_manipulator.h"
#include "utils/string_utils.h"
#include "utils/io_utils.h"

//-----------------------------------------------------------------------------
Mesh_handle Boundary_builder::build_from_mesh_file( std::ifstream & in, cell::Tissue_handle ts )
{
    if (!check_ifstream(in))
        cell_error("Reading iftream filed.");
    Geom_data gd;
    Index_on_vec_function funct(0);

    gd.read_mesh_file( in, funct );

    Mesh_handle m = new Mesh();
    process_geom_data( m, gd, ts );

    return m;
}
//-----------------------------------------------------------------------------
void Boundary_builder::process_geom_data( Mesh_handle m, Geom_data const& gd, cell::Tissue_handle ts )
{
    Mesh_manipulator manip(m);
    size_t n_cells = gd.numb_cells();
    std::vector< Node_handle > node_index_map( gd.numb_points());
    std::vector< Face_handle > face_index_map( gd.numb_faces(), 0);
    //add points to mesh as nodes
    Geom_data::Point_const_iterator p_iter, p_end;
    size_t counter;
    for ( counter = 0, p_iter = gd.points.begin(), p_end = gd.points.end(); p_iter != p_end; ++counter, ++p_iter )
    {
        node_index_map[counter] = manip.add_node( *p_iter);
    }
    //construct mesh edges and faces
    for ( size_t i = 0; i < gd.numb_faces(); ++i )
    {
        if ( face_index_map[i ] == NULL )
        {
            std::list<Node_handle> nodes;
            Geom_data::Face_point_ind_const_iterator f_iter, f_end;

            for (f_iter = gd.face_point_ind_begin( i ), f_end = gd.face_point_ind_end( i ); f_iter != f_end; ++f_iter)
            {
                Node_handle nh = node_index_map[*f_iter];
                nodes.push_back( nh );
            }
            face_index_map[i] = manip.add_polygon(nodes.begin(), nodes.end());
        }
    }
}
//-----------------------------------------------------------------------------
Mesh_handle Boundary_builder::build_mesh( std::ifstream & IN )
{
    if (!check_ifstream(IN))
        cell_error("Reading iftream filed.");
    Geom_data gd(IN);
    Mesh_handle m = new Mesh();
    process_geom_data( m, gd, NULL );

    m->set_type(Mesh::VOLUME);
    return m;
}
//-----------------------------------------------------------------------------
