// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _CELL_MESH_EDGE_H_
#define _CELL_MESH_EDGE_H_

#include "mesh/mesh_node.h"
#include "utils/flag.h"
#include "mesh/mesh_handle.h"
#include "mesh/iterator_projector_functors.h"
#include "utils/iterator_projector.h"
#include "utils/direction.h"
#include "utils/id.h"

//-----------------------------------------------------------------------------
class Mesh_edge : public Idable, public Flagable
{
public:
    typedef std::vector<Node_handle>    Node_container;
    typedef Handle_iterator< Node_container::iterator >         Node_iterator;
    typedef Handle_iterator< Node_container::const_iterator >       Node_const_iterator;

    typedef Project_point<Mesh_node>                                Point_projector;
    typedef Iterator_projector<Node_container::iterator, Point_projector>       Point_iterator;

    Mesh_edge ( Node_handle const& n0, Node_handle const& n1 ) : m_nodes ( Node_container ( 2 ) )
    {
        m_nodes[0] = n0;
        m_nodes[1] = n1;
    }

    Node_handle const& node0() const
    {
        return m_nodes[0];
    }
    Node_handle const& node1() const
    {
        return m_nodes[1];
    }
    Node_handle & node0()
    {
        return m_nodes[0];
    }
    Node_handle & node1()
    {
        return m_nodes[1];
    }

    Node_handle const& other_node ( const Node_handle &n ) const
    {
        if ( n == m_nodes[0] )
            return m_nodes[1];
        if ( n == m_nodes[1] )
            return m_nodes[0];
        return INVALID_NODE;
    }

    bool has_node ( Node_handle& n ) const
    {
        return ( ( n == m_nodes[0] ) || ( n == m_nodes[1] ) );
    }

    void set_nodes ( Node_handle& n1, Node_handle& n2 )
    {
        set_node0 ( n1 );
        set_node1 ( n2 );
    }
    void set_node0 ( Node_handle const& n )
    {
        m_nodes[0] = n;
    }
    void set_node1 ( Node_handle const& n )
    {
        m_nodes[1] = n;
    }

    void statistics ( bool det = false );

    //iterators
    Node_iterator nodes_begin()
    {
        return m_nodes.begin();
    }
    Node_const_iterator nodes_begin() const
    {
        return m_nodes.begin();
    }
    Node_iterator nodes_end()
    {
        return m_nodes.end();
    }
    Node_const_iterator nodes_end() const
    {
        return m_nodes.end();
    }

    Point_iterator points_begin()
    {
        return m_nodes.begin();
    }
    Point_iterator points_end()
    {
        return m_nodes.end();
    }

private:

    Node_container m_nodes;
    friend class Mesh_manipulator;
};
//-----------------------------------------------------------------------------
inline Direction edge_orientation ( Edge_handle e, Node_handle n1, Node_handle n2 )
{
    Node_handle &en1 = e->node0();
    Node_handle &en2 = e->node1();

    if ( en1 == n1 && en2 == n2 )
        return DIR_ORIGINAL;
    else if ( en2 == n1 && en1 == n2 )
        return DIR_REVERSED;
    else
        return DIR_INVALID;
}
//-----------------------------------------------------------------------------
#endif
