// Copyright (C) 2008 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.

#include "mesh/mesh_edge_use.h"
#include "utils/utility.h"
//-----------------------------------------------------------------------------
void Mesh_edge_use::insert_loop ( Edge_use_handle eu )
{
    //insert edge_use disconecting this and next edge_uses
    eu->m_loop_next = m_loop_next;
    m_loop_next = eu;
}
//-----------------------------------------------------------------------------
void Mesh_edge_use::insert_radial ( Edge_use_handle eu )
{
    //insert edge_use disconecting this and next edge_uses
    eu->m_radial_next = m_radial_next;
    m_radial_next = eu;
}
//-----------------------------------------------------------------------------
Mesh_edge_use::circulator Mesh_edge_use::previous_loop()
{
    circulator cir = loop_begin(), fin = cir;
    while ( fin != cir->m_loop_next )
        ++cir;
    return cir;
}
//-----------------------------------------------------------------------------
Mesh_edge_use::circulator Mesh_edge_use::previous_radial()
{
    circulator cir = radial_begin(), fin = cir;
    while ( fin != cir->m_radial_next )
        ++cir;
    return cir;
}
//-----------------------------------------------------------------------------
size_t Mesh_edge_use::size_of_loop() const
{
    const_circulator start = loop_begin(), c = start;
    size_t count = 0;
    do
    {
        ++count;

    }
    while ( ++c != start && c != INVALID_EDGE_USE );
    return count;
}
//-----------------------------------------------------------------------------
bool Mesh_edge_use::is_triangle() const
{
    const_circulator start = loop_begin(), c = start;
    size_t count = 0;
    do
    {
        if ( ++count > 3 )
            return false;
        else if ( ++c == start )
            return true;
    }
    while ( c != INVALID_EDGE_USE );
    return false;
}
//-----------------------------------------------------------------------------
size_t Mesh_edge_use::size_of_radial() const
{
    const_circulator start = radial_begin(), c = start;
    size_t count = 0;
    do
    {
        ++count;
    }
    while ( ++c != start && c != INVALID_EDGE_USE );
    return count;
}
//-----------------------------------------------------------------------------
void Mesh_edge_use::statistics ( bool detailed, std::ostream &out )
{
    out << "edge_use[" << id() << "] on nodes(" << node()->id() << ", " << other_node()->id() << ")\n";
    if ( detailed )
    {
        circulator start = loop_begin(), c = start;
        std::cout << "loop: ";
        size_t count = 0;
        do
        {
            std::cout << c->id() << "->";
            ++count;
            if ( ! ( count%20 ) )
                wait_kb_hit();
        }
        while ( ++c != start && c != INVALID_EDGE_USE );
        if ( c == INVALID_EDGE_USE )
            std::cout << "UNCLOSED";
        std::cout << "\n";

        start = radial_begin();
        c = start;
        std::cout << "radial: ";
        count = 0;
        do
        {
            std::cout << c->id() << "->";
            ++count;
            if ( ! ( count%20 ) )
                wait_kb_hit();
        }
        while ( ++c != start && c != INVALID_EDGE_USE );
        if ( c == INVALID_EDGE_USE )
            std::cout << "UNCLOSED";
        std::cout << "\n";
    }
    out << "radial degree = " << size_of_radial() << "; loop degree = " << size_of_loop() << std::endl;
}
//-----------------------------------------------------------------------------
void Mesh_edge_use::reverse()
{
    m_dir = ::reverse ( m_dir );
    if ( m_dir == DIR_REVERSED )
    {
        m_edge->node0()->remove_edge_use ( this );
        m_edge->node1()->add_edge_use ( this );
    }
    else if ( m_dir == DIR_ORIGINAL )
    {
        m_edge->node1()->remove_edge_use ( this );
        m_edge->node0()->add_edge_use ( this );
    }
    else
        cell_error ( "Unspecifed direction of edge use." );
}
//-----------------------------------------------------------------------------
