// Copyright (C) 2008 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _MESH_EDGE_USE_H_
#define _MESH_EDGE_USE_H_

#include "mesh/mesh_handle.h"
#include "mesh/mesh_edge.h"
#include "utils/id.h"
#include "utils/in_place_circle.h"
#include "utils/iterator_projector.h"
#include "mesh/iterator_projector_functors.h"
#include "utils/error_macros.h"
namespace cell
{
class Cell_manipulator;
}
//-----------------------------------------------------------------------------
class Mesh_edge_use : public Idable, public Flagable
{
public:
    typedef In_place_circulator<Edge_use_handle>       circulator;
    typedef In_place_const_circulator<Edge_use_handle> const_circulator;

    typedef Project_node_handle<Mesh_edge_use>                                             Node_projector;
    typedef Iterator_projector<In_place_circulator<Edge_use_handle>, Node_projector>       node_circulator;
    typedef Iterator_projector<In_place_const_circulator<Edge_use_handle>, Node_projector> node_const_circulator;

    Mesh_edge_use ( Edge_handle e, Direction d ) : m_edge ( e ), m_dir ( d ), m_loop_next ( this ), m_radial_next ( this )
    {
        if ( m_dir == DIR_REVERSED )
            e->node1()->add_edge_use ( this );
        else if ( m_dir == DIR_ORIGINAL )
            e->node0()->add_edge_use ( this );
        else
            cell_error ( "Invalid direction of edge use." );
    }

    Edge_handle& edge()
    {
        return m_edge;
    }
    const Edge_handle& edge() const
    {
        return m_edge;
    }

    Direction direction() const
    {
        return m_dir;
    }
    //node at which eu starts
    Node_handle const& node() const
    {
        if ( m_dir == DIR_REVERSED )
            return m_edge->node1();
        else if ( m_dir == DIR_ORIGINAL )
            return m_edge->node0();
        else
            return INVALID_NODE;
    }
    Node_handle & node()
    {
        if ( m_dir == DIR_REVERSED )
            return m_edge->node1();
        else if ( m_dir == DIR_ORIGINAL )
            return m_edge->node0();
        else
            return const_cast<Node_handle&> ( INVALID_NODE );
    }
    //node to which eu points to
    Node_handle const& other_node() const
    {
        if ( m_dir == DIR_REVERSED )
            return m_edge->node0();
        else if ( m_dir == DIR_ORIGINAL )
            return m_edge->node1();
        else
            return INVALID_NODE;
    }
    Node_handle & other_node()
    {
        if ( m_dir == DIR_REVERSED )
            return m_edge->node0();
        else if ( m_dir == DIR_ORIGINAL )
            return m_edge->node1();
        else
            return const_cast<Node_handle&> ( INVALID_NODE );;
    }

    void insert_loop ( Edge_use_handle eu );
    void insert_radial ( Edge_use_handle eu );

    size_t size_of_loop() const;
    size_t size_of_radial() const;

    bool is_triangle() const;

    circulator previous_loop();
    circulator previous_radial();

    void statistics ( bool detailed = false, std::ostream &out = std::cout );

    circulator loop_begin()
    {
        return circulator ( this, &Mesh_edge_use::loop_next );
    }
    const_circulator loop_begin() const
    {
        return const_circulator ( this, &Mesh_edge_use::loop_next );
    }
    circulator radial_begin()
    {
        return circulator ( this, &Mesh_edge_use::radial_next );
    }
    const_circulator radial_begin() const
    {
        return const_circulator ( this, &Mesh_edge_use::radial_next );
    }

    void set_loop_next ( Edge_use_handle const& eu )
    {
        m_loop_next = eu;
    }
    void set_radial_next ( Edge_use_handle const& eu )
    {
        m_radial_next = eu;
    }
    Edge_use_handle const& loop_next() const
    {
        return m_loop_next;
    }
    Edge_use_handle const& radial_next() const
    {
        return m_radial_next;
    }
private:

    void reverse();

    Edge_handle m_edge;
    Direction m_dir;

    Edge_use_handle m_loop_next;
    Edge_use_handle m_radial_next;

    friend class Mesh_manipulator;
    friend class cell::Cell_manipulator;
    friend class Triangulator;
};
//-----------------------------------------------------------------------------
#endif
