// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.

#include "mesh/mesh_face.h"

//-----------------------------------------------------------------------------
void Mesh_face::statistics ( bool detailed ) const
{
    std::cout << "Mesh face [" << id() << "] statistics:\n";
    std::cout << "Edge use:" << m_edge_use->id() << "\n";
    std::cout.flush();
    if ( detailed )
    {
        Mesh_edge_use::const_circulator start = m_edge_use->loop_begin(), c = start;
        std::cout << "loop: ";
        size_t count = 0;
        do
        {
            std::cout << c->id() << "->";
            ++count;
            if ( ! ( count%30 ) )
                wait_kb_hit();
        }
        while ( ++c != start && c != INVALID_EDGE_USE );
        if ( c == INVALID_EDGE_USE )
            std::cout << "UNCLOSED";
        std::cout << "\n";
        start = m_edge_use->loop_begin(), c = start;
        std::cout << "node loop: ";
        count = 0;
        do
        {
            std::cout << c->node()->id() << "->";
            ++count;
            if ( ! ( count%30 ) )
                wait_kb_hit();
        }
        while ( ++c != start && c != INVALID_EDGE_USE );
        if ( c == INVALID_EDGE_USE )
            std::cout << "UNCLOSED";
        std::cout << "\n";
        std::cout << "size of loop: " << count << "\n";
    }
}
//-----------------------------------------------------------------------------
void Mesh_face::check_circularity() const
{
    Edge_use_const_circulator e_iter = edge_uses_begin(), e_end = e_iter;
    Node_handle next_node, opp_node;
    next_node = e_iter->other_node();
    ++e_iter;

    do
    {
        opp_node = e_iter->node();
        if ( next_node != opp_node )
        {
            std::cout <<" face node mismatch: opp- " << opp_node->id() << ", nxt- " << next_node->id() << "\n";
            wait_kb_hit();
        }
        next_node = e_iter->other_node();
    }
    while ( ++e_iter != e_end );
}
//-----------------------------------------------------------------------------
