// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _CELL_MESH_FACE_H_
#define _CELL_MESH_FACE_H_

#include "utils/flag.h"
#include "mesh/mesh_handle.h"
#include "mesh/mesh_edge_use.h"
#include "material/generic_material.h"

#include "utils/types.h"
#include "utils/id.h"
#include "utils/direction.h"
#include "utils/in_place_list.h"
#include "fem/element_fem_data.h"

class Mesh_face : public Idable, public Flagable
{
public:

    typedef Mesh_edge_use::const_circulator             Edge_use_const_circulator;
    typedef Mesh_edge_use::circulator                   Edge_use_circulator;

public:
    Mesh_face() : m_edge_use(INVALID_EDGE_USE) {}
    Mesh_face( Edge_use_handle const& eu ) : m_edge_use(eu) {}
    ~Mesh_face() {}

    Edge_use_handle const& edge_use() const {
        return m_edge_use;
    }
    Edge_use_handle& edge_use() {
        return m_edge_use;
    }
    int size_of_edges() const {
        return m_edge_use->size_of_loop();
    }
    int size_of_nodes() const {
        return m_edge_use->size_of_loop();
    }

    bool has_edge_use( const Edge_use_handle eu ) const
    {
        Edge_use_const_circulator eu_circ = edge_uses_begin(), eu_start = eu_circ;
        do
        {
            if ( eu_circ == eu)
                return true;
        } while (++eu_circ != eu_start);
        return false;
    }

    void statistics( bool detailed = false ) const;
    void check_circularity() const;

//iterators
    Edge_use_circulator edge_uses_begin()
    {
        return m_edge_use->loop_begin();
    }
    Edge_use_const_circulator edge_uses_begin() const
    {
        return const_cast<Mesh_edge_use const&>(*m_edge_use).loop_begin();
    }

    void set_edge_use( Edge_use_handle eu ) {
        m_edge_use = eu;
    }
    Generic_material* material() const {
        return m_material;
    }
    void set_material(Generic_material* m) {
        m_material = m;
    }
    Element_fem_data& fem_data() const {
        return m_fem_data;
    }

private:
    Edge_use_handle m_edge_use;
    Generic_material *m_material;
    mutable Element_fem_data m_fem_data;
    friend class Mesh_manipulator;
};
#endif
