// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#include "mesh/mesh_node.h"
#include "mesh/mesh_edge_use.h"
//-----------------------------------------------------------------------------
Mesh_node::Mesh_node ( Node_handle n ) : m_point ( n->point() ), m_dir ( n->director() ), m_point0 ( n->point0() ), m_dir0 ( n->director0() ), m_h0 ( n->h0() )
{

}
//-----------------------------------------------------------------------------
bool Mesh_node::has_edge_use ( Edge_use_handle e ) const
{
    Edge_use_const_iterator it = edge_uses_begin(), ie = edge_uses_end();
    while ( it != ie )
    {
        if ( e == it )
            return true;
        ++it;
    }
    return false;
}
//-----------------------------------------------------------------------------
bool Mesh_node::has_edge ( Edge_handle e ) const
{
    Edge_use_const_iterator it = edge_uses_begin(), ie = edge_uses_end();
    while ( it != ie )
    {
        if ( e == it->edge() )
            return true;
        ++it;
    }
    return false;
}
//-----------------------------------------------------------------------------
void Mesh_node::remove_edge_use ( Edge_use_handle e )
{
    Edge_use_iterator it = edge_uses_begin(), ie = edge_uses_end();
    while ( it != ie )
    {
        if ( e == it )
        {
            erase_edge_use ( it.current_iterator() );
            break;
        }
        ++it;
    }
}
//-----------------------------------------------------------------------------
void Mesh_node::statistics ( bool detailed, std::ostream &out )
{
    out << "node[" << id() << "] at point (" << point() << ")\n";
    out << "degree = " << size_of_edge_uses() << "\n";
    if ( detailed )
    {
        out << "edge uses { ";
        Edge_use_const_iterator eu_it, eu_end;
        for ( eu_it = edge_uses_begin(), eu_end = edge_uses_end(); eu_it != eu_end; ++eu_it )
        {
            out << eu_it->id() << "(" << eu_it->node()->id() << "," << eu_it->other_node()->id() << ") ";
        }
        out << "}\n";
    }
}
//-----------------------------------------------------------------------------
bool Mesh_node::check_node_edges()
{
    Edge_use_const_iterator eu_it, eu_end;
    for ( eu_it = edge_uses_begin(), eu_end = edge_uses_end(); eu_it != eu_end; ++eu_it )
    {
        int i = id();
        if ( eu_it->node()->id() != i && eu_it->other_node()->id() != i )
        {
            return false;
            std::cout << "node[" << i << "] connected to the wrong edge_use\n";
            std::cout <<"[" << eu_it->id() << "](" << eu_it->node()->id() << "," << eu_it->other_node()->id() << ")\n";
        }
    }
    return true;
}
//-----------------------------------------------------------------------------
