// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _CELL_MESH_NODE_H_
#define _CELL_MESH_NODE_H_

#include <vector>
#include "utils/types.h"
#include "utils/id.h"
#include "utils/flag.h"
#include "mesh/mesh_handle.h"
//Mesh node holds the point and edge uses emanating from the node
//-----------------------------------------------------------------------------
class Mesh_node : public Idable, public Flagable
{
public:

    typedef std::vector< Edge_use_handle >                      Edge_use_container;
    typedef Handle_iterator<Edge_use_container::iterator>       Edge_use_iterator;
    typedef Handle_iterator<Edge_use_container::const_iterator> Edge_use_const_iterator;

    explicit Mesh_node ( Point const &p ) : m_point ( p ), m_dir ( 0,0,0 ), m_point0 ( 0,0,0 ), m_dir0 ( 0,0,0 ), m_h0 ( -1.0 ) {}

    Mesh_node ( Node_handle n );
    Point const& point() const
    {
        return m_point;
    }
    Point& point()
    {
        return m_point;
    }
    void set_point ( Point const& p )
    {
        m_point = p;
    };

    Vector const& director() const
    {
        return m_dir;
    }
    Vector& director()
    {
        return m_dir;
    }
    void set_director ( Vector const& p )
    {
        m_dir = p;
    };

    double h0()
    {
        return m_h0;
    }
    double set_h0 ( double h )
    {
        m_h0 = h;
    }

    Point const& point0() const
    {
        return m_point0;
    }
    Point& point0()
    {
        return m_point0;
    }
    void set_point0 ( Point const& p )
    {
        m_point0 = p;
    };

    Vector const& director0() const
    {
        return m_dir0;
    }
    Vector& director0()
    {
        return m_dir0;
    }
    void set_director0 ( Vector const& p )
    {
        m_dir0 = p;
    };

    size_t size_of_edge_uses() const
    {
        return m_edge_uses.size();
    }
    bool has_edge_use ( Edge_use_handle e ) const;
    bool has_edge ( Edge_handle e ) const;

    //iterators
    Edge_use_iterator edge_uses_begin()
    {
        return m_edge_uses.begin();
    }
    Edge_use_const_iterator edge_uses_begin() const
    {
        return m_edge_uses.begin();
    }
    Edge_use_iterator edge_uses_end()
    {
        return m_edge_uses.end();
    }
    Edge_use_const_iterator edge_uses_end() const
    {
        return m_edge_uses.end();
    }

    void statistics ( bool detailed = false, std::ostream &out = std::cout );
    int degree() const
    {
        return m_edge_uses.size();
    }
    bool check_node_edges();
private:
    void add_edge_use ( Edge_use_handle e )
    {
        m_edge_uses.push_back ( e );
    }
    Edge_use_iterator erase_edge_use ( Edge_use_iterator e )
    {
        return m_edge_uses.erase ( e.current_iterator() );
    }
    void remove_edge_use ( Edge_use_handle e );

    Edge_use_container  m_edge_uses;
    Point m_point;
    Vector m_dir;
    Point m_point0;
    Vector m_dir0;
    double m_h0;

    friend class Mesh;
    friend class Mesh_edge_use;
    friend class Mesh_manipulator;
};
//-----------------------------------------------------------------------------
#endif
