// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _VORONOI2D_H_
#define _VORONOI2D_H_

#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
#include <CGAL/Delaunay_triangulation_2.h>
#include <CGAL/Voronoi_diagram_2.h>
#include <CGAL/Delaunay_triangulation_adaptation_traits_2.h>
#include <CGAL/Delaunay_triangulation_adaptation_policies_2.h>

typedef CGAL::Exact_predicates_inexact_constructions_kernel     	VOR_K;
typedef CGAL::Delaunay_triangulation_2<VOR_K>                       VOR_DT;
typedef CGAL::Delaunay_triangulation_adaptation_traits_2<VOR_DT>    VOR_AT;
typedef CGAL::Delaunay_triangulation_caching_degeneracy_removal_policy_2<VOR_DT> VOR_AP;
typedef CGAL::Voronoi_diagram_2<VOR_DT,VOR_AT,VOR_AP>                            VOR_VD;

class Grid_base;
class VRML_ostream;
//-----------------------------------------------------------------------------
class Voronoi2D : public VOR_VD
{
public:
    // typedef for the result type of the point location
    typedef VOR_AT::Site_2							Site_2;
    typedef VOR_AT::Point_2							Point_2;
    typedef VOR_VD::Locate_result					Locate_result;
    typedef VOR_VD::Vertex_handle					Vertex_handle;
    typedef VOR_VD::Face_handle						Face_handle;
    typedef VOR_VD::Halfedge_handle					Halfedge_handle;
    typedef VOR_VD::Ccb_halfedge_circulator			Ccb_halfedge_circulator;
    typedef VOR_VD::Bounded_faces_iterator			Faces_iterator;
    typedef VOR_VD::Bounded_halfedges_iterator		Halfedges_iterator;
    typedef std::vector<Face_handle>::const_iterator		Inner_faces_iterator;
    typedef std::vector<Face_handle>::const_iterator		Outer_faces_iterator;

    enum Output_type { ALL_CELLS, INNER_CELLS, OUTER_CELLS };
    Voronoi2D ( Grid_base const& cg );
    void file_output ( VRML_ostream &o, Output_type ot = ALL_CELLS ) const;

    Faces_iterator faces_begin()
    {
        return this->bounded_faces_begin();
    }
    Faces_iterator faces_end()
    {
        return this->bounded_faces_end();
    }

    Inner_faces_iterator inner_faces_begin() const
    {
        return m_inner_cells.begin();
    }
    Inner_faces_iterator inner_faces_end() const
    {
        return m_inner_cells.end();
    }

    Outer_faces_iterator outer_faces_begin() const
    {
        return m_outer_cells.begin();
    }
    Outer_faces_iterator outer_faces_end() const
    {
        return m_outer_cells.end();
    }

    std::vector<Face_handle> m_inner_cells;
    std::vector<Face_handle> m_outer_cells;
};
//-----------------------------------------------------------------------------
#endif
