// Copyright (C) 2003-2005 Anders Logg.
// Licensed under the GNU GPL Version 2.
//
// Modified by Pawel Krupinski 2007

#ifndef __PARAMETER_H
#define __PARAMETER_H

#include<string>

class ParameterValue;

/// This class represents a parameter of some given type.
/// Supported value types are double, int, bool, and string.

class Parameter
{
public:

    /// Supported parameter types
    enum Type { type_real, type_int, type_bool, type_string };

    /// Create int-valued parameter
    Parameter ( int value );

    /// Create int-valued parameter
    Parameter ( size_t value );

    /// Create double-valued parameter
    Parameter ( double value );

    /// Create bool-valued parameter
    Parameter ( bool value );

    /// Create string-valued parameter
    Parameter ( std::string value );

    /// Create string-valued parameter
    Parameter ( const char* value );

    /// Copy constructor
    Parameter ( const Parameter& parameter );

    /// Destructor
    ~Parameter();

    /// Assignment of int
    const Parameter& operator= ( int value );

    /// Assignment of size_t
    const Parameter& operator= ( size_t value );

    /// Assignment of double
    const Parameter& operator= ( double value );

    /// Assignment of bool
    const Parameter& operator= ( bool value );

    /// Assignment of string
    const Parameter& operator= ( std::string value );

    /// Assignment of Parameter
    const Parameter& operator= ( const Parameter& parameter );

    /// Cast parameter to int
    operator int() const;

    /// Cast parameter to size_t
    operator size_t() const;

    /// Cast parameter to double
    operator double() const;

    /// Cast parameter to bool
    operator bool() const;

    /// Cast parameter to string
    operator std::string() const;

    /// Return type of parameter
    Type type() const;

    /// Friend operator <<
    friend std::ostream& operator<< ( std::ostream& stream, const Parameter& parameter );

private:

    // Pointer to parameter value
    ParameterValue* value;

    // Type of parameter
    Type _type;

};
#endif
