// Copyright (C) 2005-2006 Anders Logg.
// Licensed under the GNU GPL Version 2.
//
// Modified by Pawel Krupinski 2007

#include "parameter/ParameterSystem.h"
#include "parameter/Parametrized.h"
#include "utils/error_macros.h"

#include <string>



//-----------------------------------------------------------------------------
Parametrized::Parametrized() : parent(0)
{
  // Do nothing
}
//-----------------------------------------------------------------------------
Parametrized::~Parametrized()
{
  // Do nothing
}
//-----------------------------------------------------------------------------
void Parametrized::add(std::string key, Parameter value)
{
  parameters.add(key, value);
}
//-----------------------------------------------------------------------------
void Parametrized::set(std::string key, Parameter value)
{
  if ( !defined(key) )
    parameters.add(key, value);
  else
    parameters.set(key, value);

  readParameters();
}
//-----------------------------------------------------------------------------
void Parametrized::set(std::string key, const Parametrized& parent)
{
  // Check that key is "parent"
  if ( !(key == "parent") )
    cell_error1("Illegal value for parameter \"%s\".", key.c_str());

  // Check if we already have a parent
  if ( this->parent )
    cell_error("Local paramater database can only have one parent.");

  // Check that parent is not itself
  if ( this == &parent )
    cell_error("Local parameter database cannot be its own parent.");

  // Set parent
  this->parent = &parent;
}
//-----------------------------------------------------------------------------
Parameter Parametrized::get(std::string key) const
{
  // First check local database
  if ( defined(key) )
    return parameters.get(key);

  // Check parent if any
  if ( parent )
    return parent->get(key);

  // Fall back on global database
  return ::get(key);
}
//-----------------------------------------------------------------------------
bool Parametrized::defined(std::string key) const
{
  return parameters.defined(key);
}
//-----------------------------------------------------------------------------
void Parametrized::readParameters()
{
  // Do nothing
}
//-----------------------------------------------------------------------------
