// Copyright (C) 2008 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#include "postprocessing/postproc_utils.h"
extern "C"
{
#include "diagonalization/dsyevh3.h"
}
#include "diagonalization/eigen_sort.h"
#include "fem/principial_stress.h"
#include "postprocessing/stress_interpolator.h"
#include "mesh/mesh_handle.h"
#include "mesh/mesh_face.h"
#include "mesh/mesh_utils.h"
#include "matrix/principal_directions.h"
// #include "cell/tissue.h"
//-----------------------------------------------------------------------------
//calculating principal directions given tensor in the vector form
void calculate_principial_stress( Principal_dir & p_s, DenseVector const&s_v, bool sign )
{
    //normal vectors Q[][i] normalized
    double A[3][3], Q[3][3], w[3];

    A[0][0] = s_v(0);
    A[0][1] = s_v(5);
    A[0][2] = s_v(4);
    A[1][1] = s_v(1);
    A[1][2] = s_v(3);
    A[2][2] = s_v(2);

    int result = dsyevh3( A, Q, w );

    if ( result )
        cell_error("error in diagonalization procedure.");

    DenseMatrix &m = p_s.eigenvectors;
    DenseVector &v = p_s.vec_form;
    for ( int i = 0; i < 3; ++i )
    {
        v(i) = w[i];
        for ( int j = 0; j < 3; ++j )
        {
            m.set( i, j, Q[i][j] );
        }
    }
    if (sign)
        eigen_sort( v, m );
    else
        eigen_sort_fabs( v, m );
}
//-----------------------------------------------------------------------------
void align_material_fibers(Mesh_handle &m, State_interpolator const& si)
{
    Mesh::Face_iterator it, it_end;
    principal_directions princ;
    double eig_val[3];
    for (it = m->faces_begin(), it_end = m->faces_end(); it != it_end; ++it)
    {
        Matrix3s const& estress = si.element_stress(it.handle());
        princ.calculate_principal_directions(estress, true);
        for (size_t i=0; i<3; ++i)
            eig_val[i] = princ.val(i);
        Matrix3 const& eig_vec = princ.vec();
        assert(eig_val[0]>eig_val[1]);
        double aniso = (eig_val[0]-eig_val[1])/eig_val[0];
        if (aniso > 1)
            aniso = 1;
        else if (aniso < 0)
            aniso = 0;
        Vector3 rvec = random_vector();
        Vector3 ev0 = eig_vec.col(0);
        Vector3 ev1 = eig_vec.col(1);
        Vector3 n = ev0^ev1;
        n.unit();
        double proj = n*rvec;
        //random vector in the ev0 x ev1 plane
        Vector3 v = rvec - n*proj;
        //modeled direction of microtubule
        rvec = v + (ev0-v)*aniso;
        it->fem_data().fiber = rvec;
    }
}
//-----------------------------------------------------------------------------