// Copyright (C) 2011 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _FACE_PREDICATE_H_
#define _FACE_PREDICATE_H_

#include "predicates/predicate.h"
#include "mesh/mesh_face.h"
typedef Predicate<Mesh_face> Face_predicate;
namespace face_predicates
{
//-----------------------------------------------------------------------------
class Predicate_list: public Face_predicate
{
public:
    typedef std::list<Face_predicate*> collection;
    typedef collection::iterator iterator;
    typedef collection::const_iterator const_iterator;

    Predicate_list( ) {}

    void add(Face_predicate *np) {
        m_predicates.push_back(np);
    }

    void clear() { m_predicates.clear(); }
    bool operator()( Face_handle const& f ) const
    {
        const_iterator it, it_end;
        bool result = true;
        for (it = m_predicates.begin(), it_end = m_predicates.end(); it != it_end; ++it)
        {
            result = result && (*it)->operator()(f);
        }
        return result;
    }
private:
    collection m_predicates;
};
//-----------------------------------------------------------------------------
//'inclusive' template parameter specifies if all the face nodes have to satisfy the constraint (if true) or just one (if false)
template<class BP, bool incusive = true>
class X_position: public Face_predicate
{
public:
    X_position(double x) : m_x(x) {}
    bool operator()( Face_handle const& n ) const
    {
        bool cond = true;
        Mesh_face::Edge_use_const_circulator fe_circ = n->edge_uses_begin(), fe_start = n->edge_uses_begin();
        do {
            Point const& p = fe_circ->node()->point();
            cond = cond && BP()( p.x(), m_x);
        } while (++fe_circ != fe_start);
        return cond;
    }
private:
    double m_x;
};
//-----------------------------------------------------------------------------
template<class BP>
class X_position<BP, false>: public Face_predicate
{
public:
    X_position(double x) : m_x(x) {}
    bool operator()( Face_handle const& n ) const
    {
        Mesh_face::Edge_use_const_circulator fe_circ = n->edge_uses_begin(), fe_start = n->edge_uses_begin();
        do {
            Point const& p = fe_circ->node()->point();
            if (BP()( p.x(), m_x))
                return true;
        } while (++fe_circ != fe_start);
        return false;
    }
private:
    double m_x;
};
//-----------------------------------------------------------------------------
template<class BP, bool incusive = true>
class Y_position: public Face_predicate
{
public:
    Y_position(double y) : m_y(y) {}
    bool operator()( Face_handle const& n ) const
    {
        bool cond = true;
        Mesh_face::Edge_use_const_circulator fe_circ = n->edge_uses_begin(), fe_start = n->edge_uses_begin();
        do {
            Point const& p = fe_circ->node()->point();
            cond = cond && BP()( p.y(), m_y);
        } while (++fe_circ != fe_start);
        return cond;
    }
private:
    double m_y;
};
//-----------------------------------------------------------------------------
template<class BP>
class Y_position<BP, false>: public Face_predicate
{
public:
    Y_position(double y) : m_y(y) {}
    bool operator()( Face_handle const& n ) const
    {
        Mesh_face::Edge_use_const_circulator fe_circ = n->edge_uses_begin(), fe_start = n->edge_uses_begin();
        do {
            Point const& p = fe_circ->node()->point();
            if (BP()( p.y(), m_y))
                return true;
        } while (++fe_circ != fe_start);
        return false;
    }
private:
    double m_y;
};
//-----------------------------------------------------------------------------
template<class BP, bool incusive = true>
class Z_position: public Face_predicate
{
public:
    Z_position(double z) : m_z(z) {}
    bool operator()( Face_handle const& n ) const
    {
        bool cond = true;
        Mesh_face::Edge_use_const_circulator fe_circ = n->edge_uses_begin(), fe_start = n->edge_uses_begin();
        do {
            Point const& p = fe_circ->node()->point();
            cond = cond && BP()( p.z(), m_z);
        } while (++fe_circ != fe_start);
        return cond;
    }
private:
    double m_z;
};
//-----------------------------------------------------------------------------
template<class BP>
class Z_position<BP, false>: public Face_predicate
{
public:
    Z_position(double z) : m_z(z) {}
    bool operator()( Face_handle const& n ) const
    {
        Mesh_face::Edge_use_const_circulator fe_circ = n->edge_uses_begin(), fe_start = n->edge_uses_begin();
        do {
            Point const& p = fe_circ->node()->point();
            if (BP()( p.z(), m_z))
                return true;
        } while (++fe_circ != fe_start);
        return false;
    }
private:
    double m_z;
};
}//end namespace
//-----------------------------------------------------------------------------
#endif
