// Copyright (C) 2011 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _NODE_PREDICATES_H_
#define _NODE_PREDICATES_H_

#include "predicates/predicate.h"
#include "mesh/mesh_node.h"
typedef Predicate<Mesh_node> Node_predicate;
namespace node_predicates
{
//-----------------------------------------------------------------------------
class Predicate_list: public Node_predicate
{
public:
    typedef std::list<Node_predicate*> collection;
    typedef collection::iterator iterator;
    typedef collection::const_iterator const_iterator;

    Predicate_list( ) {}

    void add(Node_predicate *np) {
        m_predicates.push_back(np);
    }

    bool operator()( Node_handle const& n ) const
    {
        const_iterator it, it_end;
        bool result = true;
        for (it = m_predicates.begin(), it_end = m_predicates.end(); it != it_end; ++it)
        {
            result = result && (*it)->operator()(n);
        }
        return result;
    }
private:
    collection m_predicates;
};
//-----------------------------------------------------------------------------
template<class BP>
class X_position: public Node_predicate
{
public:
    X_position(double x) : m_x(x) {}
    bool operator()( Node_handle const& n ) const
    {
        return BP()( n->point().x(), m_x);
    }
private:
    double m_x;
};
//-----------------------------------------------------------------------------
template<class BP>
class Y_position: public Node_predicate
{
public:
    Y_position(double y) : m_y(y) {}
    bool operator()( Node_handle const& n ) const
    {
        return BP()( n->point().y(), m_y);
    }
private:
    double m_y;
};
//-----------------------------------------------------------------------------
template<class BP>
class Z_position: public Node_predicate
{
public:
    Z_position(double z) : m_z(z) {}
    bool operator()( Node_handle const& n ) const
    {
        return BP()( n->point().z(), m_z);
    }
private:
    double m_z;
};//-----------------------------------------------------------------------------
//tests distance of the node from the given point against BP predicate
template<class BP>
class radial_position: public Node_predicate
{
public:
    radial_position( double r, Point p = Point(0,0,0)) : m_r2(r*r), m_p(p) {}
    bool operator()( Node_handle const& n ) const
    {
        Point const& p = n->point();
        double x = m_p.x() - p.x();
        double y = m_p.y() - p.y();
        double z = m_p.z() - p.z();
        double r02 = x*x+y*y+z*z;
            return BP()( r02, m_r2);
    }
private:
    double m_r2;
    Point m_p;
};

}//end namespace
//-----------------------------------------------------------------------------
#endif
