// Copyright (C) 2011 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _POINT_PREDICATES_H_
#define _POINT_PREDICATES_H_

#include "predicates/point_predicate.h"
#include "utils/types.h"

namespace point_predicates
{
//-----------------------------------------------------------------------------
template<bool OR = false>
class Predicate_list: public Point_predicate
{
public:
    typedef std::list<Point_predicate*> collection;
    typedef collection::iterator iterator;
    typedef collection::const_iterator const_iterator;

    Predicate_list( ) {}

    void add(Point_predicate *np) {
        m_predicates.push_back(np);
    }

    bool operator()( Point const& n ) const
    {
        const_iterator it, it_end;
        bool result = true;
        for (it = m_predicates.begin(), it_end = m_predicates.end(); it != it_end; ++it)
        {
            result = result && (*it)->operator()(n);
        }
        return result;
    }
private:
    collection m_predicates;
};
//-----------------------------------------------------------------------------
template<>
inline bool Predicate_list<true>::operator()( Point const& n ) const
{
    const_iterator it, it_end;
    for (it = m_predicates.begin(), it_end = m_predicates.end(); it != it_end; ++it)
    {
        if ( (*it)->operator()(n) )
            return true;
    }
    return false;
}
//-----------------------------------------------------------------------------
template<class BP>
class X_position: public Point_predicate
{
public:
    X_position(double x) : m_x(x) {}
    bool operator()( Point const& p ) const
    {

        return BP()( p.x(), m_x);
    }
private:
    double m_x;
};
//-----------------------------------------------------------------------------
template<class BP>
class Y_position: public Point_predicate
{
public:
    Y_position(double y) : m_y(y) {}
    bool operator()( Point const& p ) const
    {

        return BP()( p.y(), m_y);
    }
private:
    double m_y;
};
//-----------------------------------------------------------------------------
template<class BP>
class Z_position: public Point_predicate
{
public:
    Z_position(double z) : m_z(z) {}
    bool operator()( Point const& p ) const
    {

        return BP()( p.z(), m_z);
    }
private:
    double m_z;
};
//-----------------------------------------------------------------------------
class True: public Point_predicate
{
public:
    True() {}
    bool operator()( Point const& p ) const
    {
        return true;
    }
};
}//end namespace
//-----------------------------------------------------------------------------
#endif
