// Copyright (C) 2011 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#include "solver/Newton_solver.h"

//-----------------------------------------------------------------------------
Newton_solver::Newton_solver(Affine_map_shell<Mesh_face> &a, Mesh_handle m) : m_aff_map(a), m_mesh(m), stress_int(m, a) {
#ifndef NDEBUG
    Mesh::Node_iterator n_iter, n_end;
    for (n_iter = m_mesh->nodes_begin(), n_end = m_mesh->nodes_end(); n_iter != n_end; ++n_iter)
    {
        if ( (n_iter->h0() < 0.0) || (n_iter->director() == Vector(0,0,0)) || (n_iter->director0() == Vector(0,0,0)) )
            cell_error("Uninitialized mesh passed to solver. Set h0 and directors first.");
    }
#endif
	add("align_material_fibers_with_stress", false);
    add("align_material_fibers_with_strain", false);
  reset();
}
//-----------------------------------------------------------------------------
void Newton_solver::reset()
{
    m_Rtol = 1.0e-5;;
    m_niter = 0;
    
    m_result = NULL;
    m_internal = NULL;
    m_R_file = NULL;
    m_load = NULL;
    m_load_increment = NULL;
    m_marked_edges = NULL;
    m_cell_outline = NULL;
    m_spreadsheet = NULL;
    m_3dplot = NULL;
    m_detailed_info = false;
    
    set("align_material_fibers_with_stress", false);
    set("align_material_fibers_with_strain", false);
}
//-----------------------------------------------------------------------------
void Newton_solver::init_step()
{
    m_niter = 0;    // nr of iterations
    m_K.zero();     // stiffness matrix
    m_Fn.zero();    // concentrated nodal force vector
    m_Fr.zero();    // nodal reaction forces
    m_ui.zero();    // displacement increment vector
    m_Ui.zero();    // Total displacement vector for iteration
    m_Ut.zero();    // Total dispalcement vector at time t (incl all previous timesteps)
    m_R1.zero();    // residual at iteration i
}
//-----------------------------------------------------------------------------
void Newton_solver::set_output(std::ofstream& file, Output_type ot)
{
    switch (ot)
    {
    case spreadsheet:
        m_spreadsheet = &file;
        break;
    case threedplot:
        m_3dplot = &file;
        break;

    default:
        cell_error("undeifned output type.");
    }
}
//-----------------------------------------------------------------------------
void Newton_solver::set_output(PVD_file &file, Output_type ot)
{
    switch (ot)
    {
    case result:
        m_result = &file;
        break;
    case internal:
        m_internal = &file;
        break;
    case residual:
        m_R_file = &file;
        break;
    case load:
        m_load = &file;
        break;
    case load_increment:
        m_load_increment = &file;
        break;
    case marked_edges:
        m_marked_edges = &file;
        break;
    case cell_outline:
        m_cell_outline = &file;
        break;
    default:
        cell_error("undeifned output type.");
    }
}
//-----------------------------------------------------------------------------
void Newton_solver::set_R_tolerance(double t)
{
    m_Rtol = t;
}
//-----------------------------------------------------------------------------
void Newton_solver::set_print_details(bool b)
{
  m_detailed_info = b;
}
//-----------------------------------------------------------------------------
