// // Copyright (C) 2007 Pawel Krupinski.
// // Licensed under the GNU GPL Version 2.
#ifndef _FORCE_LOAD_H_
#define _FORCE_LOAD_H_
#include "utils/types.h"
#include "fem/Finite_element.h"
#include "fem/Function.h"
#include "mesh/mesh.h"
//-----------------------------------------------------------------------------
//I is iterator to the mesh nodes
template<class I>
void apply_pressure_load ( Function &fun, double p, I from, I beyond )
{
    Finite_element const& elem = fun.element();
    DenseVector &vec = fun. vector();
    //number of degrees of freedom per node
    size_t *dof_index = new size_t[elem.tensordim()];
    Mesh& mesh = fun.mesh();

    vec.zero();
    while ( from != beyond )
    {
        Node_handle n = from;
        Vector &dir = n->director();
        elem.vertexmap ( dof_index, n.ptr(), mesh );
        for ( int i = 0; i < 3; ++i )
        {
            vec.add ( dof_index[i], p*dir[i] );
        }
        ++from;
    }
}
//-----------------------------------------------------------------------------
//I is iterator to the mesh faces (surface elements)
template<class I, class AFF>
void apply_pressure_load ( Function &fun, double p, I from, I beyond, AFF &map )
{
    typedef typename AFF::Mesh_element Surface_face;
    Finite_element &elem = map.element();
    //number of integration points
    size_t nint = map.q_rule().spacedim();
    //number of total degrees of freedom
    size_t ntotd = map.element().spacedim();
    //number of degrees of freedom per node
    size_t ned = map.element().tensordim();
    //number of element nodes
    double *wgts = new double[nint];
    map.q_rule().weights ( wgts );
    size_t nen = ntotd/ned;
    DenseVector &vec = fun. vector();
    size_t *dof_index = new size_t[ntotd];
    Mesh& mesh = fun.mesh();
    vec.zero();

    while ( from != beyond )
    {
        Surface_face const& f = *from;
        map.set_flag ( AFF::UPDATE_NONE );
        map.update ( f );
        map.nodemap ( dof_index, f, mesh );

        // repeat over integration points
        for ( int n=0; n<nint; ++n )
        {
            double tr = 0;
            Vector3 dxr = Vector3 ( 0,0,0 );
            Vector3 dxs = Vector3 ( 0,0,0 );
            for ( int i=0; i<nen; ++i )
            {
                Point const& P0 = map.P0 ( i );
                Vector3 r0i = Vector3 ( P0.x(), P0.y(), P0.z() );
                tr += map.shape_fun ( n,i ) *p;
                dxr += r0i*map.shape_fun_der_x ( n,i );
                dxs += r0i*map.shape_fun_der_y ( n,i );;
            }

            Vector3 f = ( dxr ^ dxs ) *tr*wgts[n];
            Vector dir ( f.x, f.y, f.z );
            for ( int i=0; i<nen; ++i )
            {
                for ( int j = 0; j < 3; ++j )
                {
                    vec.add ( dof_index[i*nen+j], p*dir[j] );
                }
            }
        }

        ++from;
    }
}
//-----------------------------------------------------------------------------
template<class I>
void apply_node_load ( Function &fun, Vector v, I from, I beyond )
{
    Finite_element &elem = fun.element();
    DenseVector &vec = fun. vector();
    size_t *dof_index = new size_t[elem.spacedim()];
    Mesh& mesh = fun.mesh();

    vec.zero();
    while ( from != beyond )
    {
        elem.vertexmap ( dof_index, from->ptr(), mesh );
        for ( int i = 0; i < 3; ++i )
        {
            vec.add ( dof_index[i], v[i] );
        }
        ++from;
    }
}
//-----------------------------------------------------------------------------
#endif
