// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _CELL_FUNCTION_OBJECTS_H_
#define _CELL_FUNCTION_OBJECTS_H_

#include "utils/types.h"

class Mesh_node;
class Mesh_edge;
class Mesh_face;

//-----------------------------------------------------------------------------
//predictates
template < class T >
struct less_id
{
    typedef T				argument_type;
    bool    operator() ( T const& t0, T const& t1 )  const
    {
        return t0.id() < t1.id();
    }
};
//-----------------------------------------------------------------------------
template < class T >
struct Compare_pointer
{
    typedef T				argument_type;
    bool    operator() ( T const *t_p, T const& t )  const
    {
        return &t == t_p;
    }
    bool    operator() ( T const& t, T const *t_p )  const
    {
        return &t == t_p;
    }
};
//-----------------------------------------------------------------------------
template < class T >
struct Compare_pointers
{
    typedef T				argument_type;
    bool    operator() ( T const & t1, T const& t2 )  const
    {
        return &t1 == &t2;
    }
};
//-----------------------------------------------------------------------------
template < class T >
struct Equal
{
    typedef T				argument_type;
    bool    operator() ( T const& t1, T const& t2 ) const
    {
        return t1 == t2;
    }
};
//-----------------------------------------------------------------------------
template < class BC_evaluator_element >
struct Project_BC_node
{
    typedef BC_evaluator_element	argument_type;
    typedef Mesh_node				result_type;

    Mesh_node& operator() ( BC_evaluator_element& x )
    {
        return  const_cast<Mesh_node&> ( * ( x.m_node_p ) ) ;
    }
    Mesh_node const& operator() ( const BC_evaluator_element& x )	const
    {
        return * ( x.m_node_p );
    }
};
//-----------------------------------------------------------------------------
template < class Insert_pair, class ME >
struct Project_BC_element
{
    typedef Insert_pair	argument_type;
    typedef ME	result_type;

    result_type& operator() ( argument_type& x )
    {
        return * ( x.first ) ;
    }
    const result_type& operator() ( const argument_type& x )	const
    {
        return * ( x.first );
    }
};
//-----------------------------------------------------------------------------
#endif
