// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _FUNCTION_ON_INDICES_H_
#define _FUNCTION_ON_INDICES_H_

#include <vector>
//-----------------------------------------------------------------------------
template< class Val >
class Function_on_indices
{
public:
    typedef Val			Value_type;

    Function_on_indices ( size_t n, Val const& v = Val() );

    void clear ( Val const& v = Val() );

    void set ( size_t n, Val const& val )
    {
        m_values[n] = val;
    }

    void resize ( size_t n )
    {
        m_values.resize ( n );
    }

    Val const& get ( size_t n ) const
    {
        if ( m_values.size() <= n ) return m_def_val;
        else return m_values[n];
    }

    Val const& operator() ( size_t n ) const
    {
        if ( m_values.size() <= n ) return m_def_val;
        else return m_values[n];
    }

    size_t size() const
    {
        return m_values.size();
    }

private:
    std::vector< Val > m_values;
    Val m_def_val;
};
//-----------------------------------------------------------------------------
template< class Val >
Function_on_indices<Val>::Function_on_indices ( size_t n, Val const& v ) : m_def_val ( v )
{
    m_values.resize ( n );
}
//-----------------------------------------------------------------------------
template< class Val >
void  Function_on_indices<Val>::clear ( Val const& v )
{
    m_values.clear();
    m_def_val = v;
}
//-----------------------------------------------------------------------------
#endif
