// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _GEOMETRY_UTILS_H_
#define _GEOMETRY_UTILS_H_

#include "utils/types.h"
#include "utils/constants.h"
#include "utils/utility.h"
#include <vector>
#include <matrix/Vector3.h>

//-----------------------------------------------------------------------------
template <class Facet>
struct Plane_equation
{
    typename Facet::Plane operator()( Facet& f) {
        typename Facet::Halfedge_handle h = f.halfedge();
        typedef typename Facet::Plane Plane;
        return Plane( h->vertex()->point(),
                      h->next()->vertex()->point(),
                      h->next()->next()->vertex()->point());
    }
};
//
//-----------------------------------------------------------------------------
struct Move_towards
{
    enum Type
    {
        absolute,
        relative
    };

    Point &_dest;
    double _factor;
    Type type;

    Move_towards(Point dest, double factor, Type tp): _dest( dest ), _factor( factor ), type(tp) {};
    Point operator()( Point pt)
    {
        Vector vec( pt, _dest);
        if (type == absolute)
            vec = vec/sqrt(vec*vec);
        return pt + ( vec * _factor );
    }
};
//-----------------------------------------------------------------------------
class Cut_plane
{
public:
    Cut_plane( Bbox const& bbox, Plane const& pl): m_bbox(bbox), m_plane(pl) {}
    std::vector< Point > get_cut_quad()
    {
        std::vector<Point> ret_points;
        ret_points.reserve(4);


        return ret_points;
    }
private:
    Bbox m_bbox;
    Plane m_plane;
};
//-----------------------------------------------------------------------------
class Winding_number
{
public:
    template< class Iter>
    double operator()( Point p, Iter pi_beg, Iter pi_end, Vector normal )
    {
        double theta = 0;
        //set iterators for previous and current points
        Iter prev = pi_beg, curr = pi_beg;
        ++curr;
        //calculate the winding number
        while ( curr != pi_end )
        {
            Vector v1( p, prev );
            Vector v2( p, curr );
            theta += asin ( cross_product( v1, v2 ) * normal ) /
                     sqrt( squared_distance( p, prev ) * squared_distance( p, curr ) );
            ++prev;
            ++curr;
        }

        //
        Vector v1( p, prev );
        Vector v2( p, pi_beg );
        theta += asin ( cross_product( v1, v2 ) * normal ) /
                 sqrt( squared_distance( p, prev ) * squared_distance( p, pi_beg ) );

        return fabs( theta );
    }
};
//-----------------------------------------------------------------------------
inline void normalize(Vector & v)
{
    double n = sqrt(v.squared_length());
    v = v/n;
}
//-----------------------------------------------------------------------------
inline Vector normalized(Vector const& v)
{
    return v/sqrt(v.squared_length());
}
//-----------------------------------------------------------------------------
//return unit vector in random direction (naive implementation, random box distribution)
inline Vector3 random_vector(bool zzero = false)
{
    double x = rand()/(RAND_MAX + 1.0);
    double y = rand()/(RAND_MAX + 1.0);
    double z = 0.0;
    if (!zzero)
        z = rand()/(RAND_MAX + 1.0);
    double norm = sqrt(x*x+y*y+z*z);
    return Vector3(x/norm, y/norm, z/norm);
}
//-----------------------------------------------------------------------------
#endif
