// Copyright (C) 2008 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _CELL_HANDLE_H_
#define _CELL_HANDLE_H_

#include "utils/handle_iterator.h"
#include "utils/in_place_circle.h"
//-----------------------------------------------------------------------------
template<class T>
class Handle
{

public:
    typedef T               value_type;
    typedef T*              pointer;
    typedef T&              reference;
    typedef std::size_t     size_type;
    typedef std::ptrdiff_t  difference_type;
    typedef std::random_access_iterator_tag iterator_category;

    Handle ( T *p = NULL ) : m_pointer ( p ) {}
    Handle ( Handle<T> const& h ) : m_pointer ( h.m_pointer ) {}
    template<class I>
    Handle ( Handle_iterator<I> const& hi ) : m_pointer ( const_cast<pointer>( hi.ptr() ) ) {}
    template<class I>
    Handle ( In_place_circulator<I> const& hc ) : m_pointer ( const_cast<pointer>( hc.ptr() ) ) {}

    pointer ptr()
    {
        return m_pointer;
    }
    const pointer ptr() const
    {
        return m_pointer;
    }

    reference operator *() const
    {
        return *m_pointer;
    }
    pointer operator ->() const
    {
        return m_pointer;
    }

    Handle& operator= ( const Handle &rhs )
    {
        m_pointer = rhs.m_pointer;
        return *this;
    }

    bool operator == ( const T* p ) const
    {
        return m_pointer == p;
    }
    bool operator == ( const Handle& h ) const
    {
        return m_pointer == h.m_pointer;
    }
    bool operator != ( const Handle& h ) const
    {
        return m_pointer != h.m_pointer;
    }
    bool operator< ( const Handle& h ) const
    {
        return id() <  h.id();
    }

private:
    int id() const
    {
        return m_pointer->id();
    }
    T * m_pointer;
    template<class TT>
    friend std::ostream& operator<< ( std::ostream& os, const Handle<TT>& h );
};
//-----------------------------------------------------------------------------
template<class T>
inline std::ostream& operator<< ( std::ostream& os, const Handle<T>& h )
{
    if ( h.operator ->() )
        return ( os << ( unsigned long ) h.operator ->() << ":[" << h.id() << "]" );
    else
        return ( os << ( unsigned long ) h.operator ->() << ":[-1]" );
}
//-----------------------------------------------------------------------------
#endif
