// Copyright (C) 2008 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _HANDLE_ITERATOR_H_
#define _HANDLE_ITERATOR_H_
//-----------------------------------------------------------------------------
template < class I >
class Handle_iterator {
    //-----------------------------------------------------------------------------
    // If T === T1 return R1 else return R2
    template <class T, class T1, class R1, class R2>
    struct I_TYPE_MATCH_IF {
        typedef R2 Result;
    };  // else clause
    template <class T, class R1, class R2>
    struct I_TYPE_MATCH_IF<T,T,R1,R2> {
        typedef R1 Result;
    }; // then clause
    //-----------------------------------------------------------------------------
protected:
    I        m_internal;
public:
    typedef Handle_iterator<I>                  Self;
    typedef I                                   Iterator;
    typedef std::iterator_traits<I>             traits;
    typedef typename traits::difference_type    difference_type;
    typedef typename traits::iterator_category  iterator_category;
    typedef typename traits::value_type         base_value_type;
    typedef typename traits::pointer            base_pointer;
    typedef typename traits::reference          base_reference;

    typedef typename base_value_type::value_type    value_type;

    typedef I_TYPE_MATCH_IF< base_reference, const base_value_type &, const value_type &, value_type &> Match1;
    typedef typename Match1::Result reference;

    typedef I_TYPE_MATCH_IF< base_pointer, const base_value_type *, const value_type *, value_type *> Match2;
    typedef typename Match2::Result pointer;

    Handle_iterator() {}
    Handle_iterator( I j) : m_internal(j) {}

    template <class I2>
    Handle_iterator( const Handle_iterator<I2> &i2): m_internal( i2.current_iterator()) {}

    template <class I2>
    Self& operator= ( const Handle_iterator<I2> &i2)
    {
        m_internal = i2.current_iterator();
        return *this;
    }

    base_value_type handle() const {
        return *m_internal;
    }
    Iterator  current_iterator() const {
        return m_internal;
    }
    pointer   ptr() const {
        return m_internal->operator->();
    }
    bool operator==( const Self& i) const {
        return ( m_internal == i.m_internal);
    }
    bool operator!=( const Self& i) const {
        return !(*this == i);
    }
    bool operator==( const base_value_type& i) const {
        return (handle() == i);
    }
    bool operator!=( const base_value_type& i) const {
        return !(handle() == i);
    }
    reference operator* () const {
        return *ptr();
    }
    pointer operator->() const {
        return ptr();
    }
    Self& operator++()
    {
        ++m_internal;
        return *this;
    }

    Self operator++(int)
    {
        Self tmp = *this;
        ++*this;
        return tmp;
    }

    Self& operator--()
    {
        --m_internal;
        return *this;
    }
    Self  operator--(int)
    {
        Self tmp = *this;
        --*this;
        return tmp;
    }

    Self& operator+=( difference_type n)
    {
        m_internal += n;
        return *this;
    }
    Self  operator+( difference_type n) const
    {
        Self tmp = *this;
        return tmp += n;
    }
    Self& operator-=( difference_type n)
    {
        return operator+=( -n);
    }
    Self  operator-( difference_type n) const
    {
        Self tmp = *this;
        return tmp += -n;
    }
    difference_type  operator-( const Self& i) const {
        return m_internal - i.m_internal;
    }
    reference  operator[]( difference_type n) const
    {
        Self tmp = *this;
        tmp += n;
        return tmp.operator*();
    }
    bool operator< ( const Self& i) const {
        return ( m_internal < i.m_internal);
    }
    bool operator> ( const Self& i) const {
        return i < *this;
    }
    bool operator<=( const Self& i) const {
        return !(i < *this);
    }
    bool operator>=( const Self& i) const {
        return !(*this < i);
    }
};
//-----------------------------------------------------------------------------
template < class Dist, class I >
inline Handle_iterator<I> operator+( Dist n, Handle_iterator<I> i)
{
    return i += n;
}
//-----------------------------------------------------------------------------
#endif
