// Copyright (C) 2008 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _IN_PLACE_CIRCLE_H_
#define _IN_PLACE_CIRCLE_H_

//Class describes circularly connected list of handles. Iterators are assumed to be handle iterators.
//-----------------------------------------------------------------------------
template<class T >
class In_place_circulator
{
public:
    typedef typename T::value_type    value_type;
    typedef typename T::pointer       pointer;
    typedef typename T::reference     reference;
    typedef std::size_t               size_type;
    typedef std::ptrdiff_t            difference_type;
    typedef std::forward_iterator_tag iterator_category;
    typedef In_place_circulator<T>    Self;

    In_place_circulator() {}
    In_place_circulator ( const Self& x ) :  m_node ( x.m_node ), m_functor ( x.m_functor ) {}
    explicit In_place_circulator ( pointer const pos, T const& ( value_type::*f ) () const /*T&(value_type::*f)()*/ ) : m_node ( T ( pos ) ), m_functor ( f ) {}

    bool  operator== ( const Self& x ) const
    {
        return m_node == x.m_node;
    }
    bool  operator!= ( const Self& x ) const
    {
        return m_node != x.m_node;
    }
    bool  operator== ( const T& x ) const
    {
        return m_node == x;
    }
    bool  operator!= ( const T& x ) const
    {
        return m_node != x;
    }

    reference operator*() const
    {
        return *m_node;
    }
    pointer operator->() const
    {
        return  m_node.operator->();
    }

    T const& handle() const
    {
        return m_node;
    }

    T& handle()
    {
        return m_node;
    }

    pointer ptr() const
    {
        return m_node.operator->();
    }

    Self& operator++()
    {
        m_node = ( ( *m_node ).*m_functor ) ();
        return *this;
    }

    Self  operator++ ( int )
    {
        Self tmp = *this;
        ++*this;
        return tmp;
    }
private:
    //Handle T is local to the circulator
    T m_node;
    T const& ( value_type::*m_functor ) () const;
    template<class>
    friend class In_place_const_circulator;
};
//-----------------------------------------------------------------------------
template<class T>
class In_place_const_circulator
{
private:
    typedef typename T::value_type base_value_type;
    typedef typename T::pointer    base_pointer;
    typedef typename T::reference  base_reference;
public:
    typedef const typename T::value_type value_type;
    typedef value_type*         pointer;
    typedef value_type&         reference;
    typedef std::size_t                  size_type;
    typedef std::ptrdiff_t               difference_type;
    typedef std::forward_iterator_tag    iterator_category;
    typedef In_place_const_circulator<T> Self;
    typedef In_place_circulator<T>  No_const_self;

    In_place_const_circulator() {}
    In_place_const_circulator ( const Self& x ) :  m_node ( x.m_node ), m_functor ( x.m_functor ) {}
    In_place_const_circulator ( No_const_self const& x ) :  m_node ( x.m_node ), m_functor ( x.m_functor ) {}
    explicit In_place_const_circulator ( pointer pos, T const& ( value_type::*f ) () const ) : m_node ( T ( const_cast<base_pointer> ( pos ) ) ), m_functor ( f ) {}

    bool  operator== ( No_const_self const& x ) const
    {
        return m_node == x.m_node;
    }
    bool  operator!= ( No_const_self const& x ) const
    {
        return m_node != x.m_node;
    }
    bool  operator== ( const Self& x ) const
    {
        return m_node == x.m_node;
    }
    bool  operator!= ( const Self& x ) const
    {
        return m_node != x.m_node;
    }

    bool  operator== ( const T& x ) const
    {
        return m_node == x;
    }
    bool  operator!= ( const T& x ) const
    {
        return m_node != x;
    }

    reference operator*() const
    {
        return *m_node;
    }
    pointer const operator->() const
    {
        return  m_node.operator->();
    }

    T const& handle() const
    {
        return m_node;
    }
    pointer ptr() const
    {
        return m_node.operator->();
    }
    Self& operator++()
    {
        m_node = ( ( *m_node ).*m_functor ) ();
        return *this;
    }

    Self  operator++ ( int )
    {
        Self tmp = *this;
        ++*this;
        return tmp;
    }
private:
    //handle T is local to the circulator
    T m_node;
    T const& ( value_type::*m_functor ) () const;
};
#endif
