// Copyright (C) 2008 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _IN_PLACE_LIST_H_
#define _IN_PLACE_LIST_H_
//In_place_list is a single connected list of handles. Iterators dereference Handle types.
//Template argument should be Handle like type.
//-----------------------------------------------------------------------------
template <class T> class In_place_list_iterator;
template <class T> class In_place_list_const_iterator;
template <class T> class In_place_list;
//-----------------------------------------------------------------------------
template < class T > class In_place_sl_list_base
{
public:
    T m_next;
};
//-----------------------------------------------------------------------------
template < class T > class In_place_list_base
{
public:
    T m_next;
    T m_prev;

};
//-----------------------------------------------------------------------------
template <class T> class In_place_list_iterator
{
protected:
    T node;
public:
    friend  class In_place_list<T>;

    typedef In_place_list_iterator<T>  Self;
    typedef In_place_list_base<T>      Base;

    typedef typename T::value_type              value_type;
    typedef typename T::pointer                 pointer;
    typedef typename T::reference               reference;
    typedef std::size_t         size_type;
    typedef std::ptrdiff_t              difference_type;
    typedef std::bidirectional_iterator_tag   iterator_category;

    In_place_list_iterator() : node(0) {}
    In_place_list_iterator(T& x) : node(x) {}

    bool  operator==( const Self& x) const {
        return node == x.node;
    }
    bool  operator!=( const Self& x) const {
        return node != x.node;
    }
    reference operator*() const {
        return *node;
    }
    T operator->() const {
        return  node;
    }

    Self& operator++()
    {
        node = node->m_next;
        return *this;
    }

    Self  operator++(int)
    {
        Self tmp = *this;
        ++*this;
        return tmp;
    }

    Self& operator--()
    {
        node = node->m_prev;
        return *this;
    }

    Self  operator--(int)
    {
        Self tmp = *this;
        --*this;
        return tmp;
    }
};
//-----------------------------------------------------------------------------
template <class T> class In_place_list_const_iterator
{
protected:
    const T node;
public:
    friend  class In_place_list<T>;

    typedef In_place_list_const_iterator<T>     Self;
    typedef In_place_list_iterator<T>                   Iterator;
    typedef In_place_list_base<T>                       Base;

    typedef const typename T::value_type        value_type;
    typedef const typename T::pointer           pointer;
    typedef const typename T::reference         reference;
    typedef std::size_t                 size_type;
    typedef std::ptrdiff_t                      difference_type;
    typedef std::forward_iterator_tag   iterator_category;

    In_place_list_const_iterator() : node(0) {}
    In_place_list_const_iterator( Iterator i) : node(&*i) {}
    In_place_list_const_iterator(const T x) : node(x) {}

    bool operator==( const Self& x) const {
        return node == x.node;
    }
    bool operator!=( const Self& x) const {
        return node != x.node;
    }

    reference operator*() const {
        return *node;
    }
    const T operator->() const {
        return  node;
    }

    Self& operator++()
    {
        node = node->m_next;
        return *this;
    }

    Self operator++(int)
    {
        Self tmp = *this;
        ++*this;
        return tmp;
    }
};
//-----------------------------------------------------------------------------
template <class T> class In_place_list
{


public:

    typedef typename T::value_type          value_type;
    typedef typename T::pointer             pointer;
    typedef const typename T::pointer       const_pointer;
    typedef typename T::reference           reference;
    typedef const typename T::reference     const_reference;
    typedef typename T::size_type           size_type;
    typedef typename T::difference_type     difference_type;

    typedef In_place_list_iterator<T> iterator;
    typedef In_place_list_const_iterator<T> const_iterator;

    typedef In_place_list<T>          Self;

protected:
    pointer      node;
    size_type    length;


public:
    explicit In_place_list() : length(0)
    {
        node = T();
        (*node).m_next = node;
    }

    void swap(Self& x)
    {
        std::swap(node, x.node);
        std::swap(length, x.length);
    }

    iterator       begin() {
        return node->m_next;
    }
    const_iterator begin() const {
        return node->m_next;
    }
    iterator       end() {
        return node;
    }
    const_iterator end() const {
        return node;
    }

    bool            empty() const    {
        return length == 0;
    }
    size_type       size() const     {
        return length;
    }

    reference       front()          {
        return *begin();
    }
    const_reference front() const    {
        return *begin();
    }
    reference       back()           {
        return *(--end());
    }
    const_reference back() const     {
        return *(--end());
    }

    iterator insert(iterator position, T& x)
    {

        x.m_next = position.node;
        (*((*position.node).m_prev)).m_next = &x;
        ++length;
        return &x;
    }

    iterator insert(T* pos, T& x)
    {
        return insert( iterator(pos), x);
    }

    void push_front(T& x) {
        insert(begin(), x);
    }
    void push_back(T& x)  {
        insert(end(), x);
    }
    void insert(iterator position, size_type n);
    void insert(iterator position, size_type n, const T& x);
    void insert( T* pos, size_type n) {
        insert( iterator(pos), n);
    }
    void insert( T* pos, size_type n, const T& x)
    {
        insert( iterator(pos), n, x);
    }

    template <class InputIterator>
    void insert(iterator pos, InputIterator first, InputIterator last)
    {
        while (first != last)
            insert(pos, *get_node(*first++));
    }

    template <class InputIterator>
    void insert(T* pos, InputIterator first, InputIterator last)
    {
        while (first != last)
            insert(pos, *get_node(*first++));
    }

    void insert(T* pos, const T* first, const T* last)
    {
        insert( iterator(pos), const_iterator(first),
                const_iterator(last));
    }
    explicit In_place_list(size_type n, const T& value) : length(0)
    {
        node = T();
        (*node).m_next = node;
        (*node).m_prev = node;
        insert(begin(), n, value);
    }
    explicit In_place_list(size_type n) : length(0)
    {
        node = T();
        (*node).m_next = node;
        (*node).m_prev = node;
        T t;
        insert(begin(), n, t);
    }

    template <class InputIterator>
    In_place_list( InputIterator first, InputIterator last) : length(0)
    {
        node = T();
        (*node).m_next = node;
        (*node).m_prev = node;
        insert( begin(), first, last);
    }

    In_place_list(const T* first, const T* last) : length(0)
    {
        node = T();
        (*node).m_next = node;
        (*node).m_prev = node;
        insert(begin(), first, last);
    }
    In_place_list(const Self& x) : length(0)
    {
        node = T();
        (*node).m_next = node;
        (*node).m_prev = node;
        insert(begin(), x.begin(), x.end());
    }
    ~In_place_list()
    {
        erase(begin(), end());
        put_node(node);
    }

    Self& operator=(const Self& x);

    void destroy();

    template <class InputIterator>
    void assign( InputIterator first, InputIterator last)
    {
        erase( begin(), end());
        insert( begin(), first, last);
    }

    void assign( size_type n, const T& t)
    {
        erase( begin(), end());
        insert( begin(), n, t);
    }

    void resize( size_type sz, const T& c)
    {
        if ( sz > size())
            insert( end(), sz - size(), c);
        else if ( sz < size())
        {
            iterator i = begin();
            while ( sz-- > 0)
                ++i;
            erase( i, end());
        }
    }
    void resize( size_type sz)
    {
        T t;
        resize( sz, t);
    }
    bool operator==( const Self& y) const
    {
        return size() == y.size() && std::equal(begin(), end(), y.begin());
    }

    bool operator!=( const Self& y) const
    {
        return size() != y.size() || ! std::equal(begin(),end(),y.begin());
    }

    bool operator<(const Self& y) const
    {
        return std::lexicographical_compare( begin(),end(),
                                             y.begin(),y.end());
    }
    bool operator> ( const Self& i) const {
        return i < *this;
    }
    bool operator<=( const Self& i) const {
        return !(i < *this);
    }
    bool operator>=( const Self& i) const {
        return !(*this < i);
    }
protected:
    void transfer(iterator position, iterator first, iterator last) {
        (*((*last.node).m_prev)).m_next = position.node;
        (*((*first.node).m_prev)).m_next = last.node;
        (*((*position.node).m_prev)).m_next = first.node;
        T* tmp = (*position.node).m_prev;
        (*position.node).m_prev = (*last.node).m_prev;
        (*last.node).m_prev = (*first.node).m_prev;
        (*first.node).m_prev = tmp;
    }

public:
    void splice(iterator position, Self& x)
    {
        if (!x.empty()) {
            transfer(position, x.begin(), x.end());
            length += x.length;
            x.length = 0;
        }
    }
    void splice(T* position, Self& x)
    {
        splice( iterator(position), x);
    }
    void splice( iterator position, Self& x, iterator i) {
        iterator j = i;
        if (position == i || position == ++j) return;
        transfer(position, i, j);
        ++length;
        --x.length;
    }
    void splice(T* position, Self& x, T* i)
    {
        splice( iterator(position), x, iterator(i));
    }
    void splice(iterator pos, Self& x, iterator first, iterator last)
    {
        if (first != last)
        {
            if (&x != this)
            {
                difference_type n = std::distance(first, last);
                x.length -= n;
                length += n;
            }
            transfer(pos, first, last);
        }
    }
    void splice(T* p, Self& x, T* first, T* last)
    {
        splice( iterator(p), x, iterator(first), iterator(last));
    }

    void remove(const T& value);
    void reverse();
    void unique();
    void merge(Self& x);
    template < class StrictWeakOrdering >
    void merge(Self& x, StrictWeakOrdering ord)
    {
        iterator first1 = begin();
        iterator last1 = end();
        iterator first2 = x.begin();
        iterator last2 = x.end();
        while (first1 != last1 && first2 != last2)
            if (ord(*first2, *first1))
            {
                iterator next = first2;
                transfer(first1, first2, ++next);
                first2 = next;
            } else
                ++first1;
        if (first2 != last2)
            transfer(last1, first2, last2);
        length += x.length;
        x.length= 0;
    }

    void sort();
//-----------------------------------------------------------------------------
    template < class StrictWeakOrdering > void sort(StrictWeakOrdering ord)
    {
        if (size() < 2) return;
        In_place_list<T> carry;
        In_place_list<T> counter[64];
        int fill = 0;
        while (!empty())
        {
            carry.splice(carry.begin(), *this, begin());
            int i = 0;
            while (i < fill && !counter[i].empty())
            {
                counter[i].merge(carry, ord);
                carry.swap(counter[i++]);
            }
            carry.swap(counter[i]);
            if (i == fill)
                ++fill;
        }
        for (int i = 1; i < fill; ++i)
            counter[i].merge(counter[i-1], ord);
        swap(counter[fill-1]);
    }

};
//-----------------------------------------------------------------------------
template <class T>
void In_place_list<T>::insert(In_place_list_iterator<T> position, size_type n)
{
    while (n--)
        insert(position, T());
}
//-----------------------------------------------------------------------------
template <class T>
void In_place_list<T>::insert(In_place_list_iterator<T> position, size_type n, const T& x)
{
    while (n--)
        insert(position, T(x));
}

//-----------------------------------------------------------------------------
template <class T>
In_place_list<T>& In_place_list<T>::operator=(const In_place_list<T>& x)
{
    if (this != &x)
    {
        iterator first1 = begin();
        iterator last1  = end();
        const_iterator first2 = x.begin();
        const_iterator last2  = x.end();
        while (first1 != last1 && first2 != last2)
        {
            T* tmp1 = (*first1).m_next;
            T* tmp2 = (*first1).m_prev;
            *first1 = *first2++;
            (*first1).m_next = tmp1;
            (*first1).m_prev = tmp2;
            ++first1;
        }
        if (first2 == last2)
            erase(first1, last1);
        else
            insert(last1, first2, last2);
    }
    return *this;
}
//-----------------------------------------------------------------------------
template <class T>
void In_place_list<T>::reverse()
{
    if (size() < 2) return;
    for (iterator first = ++begin(); first != end();)
    {
        iterator old = first++;
        transfer(begin(), old, first);
    }
}
//-----------------------------------------------------------------------------
template <class T>
void In_place_list<T>::unique()
{
    iterator first = begin();
    iterator last = end();
    if (first == last) return;
    iterator next = first;
    while (++next != last)
    {
        if (*first == *next)
            erase(next);
        else
            first = next;
        next = first;
    }
}
//-----------------------------------------------------------------------------
template <class T> void In_place_list<T>::merge(In_place_list<T>& x)
{
    iterator first1 = begin();
    iterator last1 = end();
    iterator first2 = x.begin();
    iterator last2 = x.end();
    while (first1 != last1 && first2 != last2)
        if (*first2 < *first1)
        {
            iterator next = first2;
            transfer(first1, first2, ++next);
            first2 = next;
        } else
            ++first1;
    if (first2 != last2)
        transfer(last1, first2, last2);
    length += x.length;
    x.length= 0;
}
//-----------------------------------------------------------------------------
template <class T> void In_place_list<T>::sort()
{
    if (size() < 2) return;
    In_place_list<T> carry;
    In_place_list<T> counter[64];
    int fill = 0;
    while (!empty())
    {
        carry.splice(carry.begin(), *this, begin());
        int i = 0;
        while (i < fill && !counter[i].empty())
        {
            counter[i].merge(carry);
            carry.swap(counter[i++]);
        }
        carry.swap(counter[i]);
        if (i == fill)
            ++fill;
    }
    for (int i = 1; i < fill; ++i)
        counter[i].merge(counter[i-1]);
    swap(counter[fill-1]);
}
//-----------------------------------------------------------------------------
#endif
