// Copyright (C) 2009 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _INTERSECTION_POINT_H_
#define _INTERSECTION_POINT_H_

#include "utils/constants.h"
#undef DEBUG_PRINT
// #define DEBUG_PRINT
//-----------------------------------------------------------------------------
//determines if plane given by normal vector and a point intersects a segment given by two points. If so intersection point is writen to supplied Point reference
//code==-1 - segment in plane
//code==0 - no division
//code==1 - division in the middle of segment
//code==2 - division at p1 of current segment
//code==3 - division at p2 of current segment
class Intersection_point
{
public:
    int operator()( Point &p, Vector const& n, Point & p1, Point & p2 )
    {
        Vector w = p - p1;
        Vector w2 = p - p2;
        Vector u = p2 - p1;
        double u_len2 = u.squared_length();
        double const PROX_SQR = PROXIMITY_THRESHOLD*PROXIMITY_THRESHOLD;

        double den = n*u;
        double nom = n*w;
        double nom2 = n*w2;
        //check if both points are within proximity threshold
        if ((nom < PROXIMITY_THRESHOLD && nom > -PROXIMITY_THRESHOLD)&&(nom2 < PROXIMITY_THRESHOLD && nom2 > -PROXIMITY_THRESHOLD))
        {
            p1 = p1+n*nom;
            p2 = p2+n*nom2;
            return -1;
        }
        //else if the plane is parallel (within threshold) to the segment it is disjoint
        if ( den < PROXIMITY_THRESHOLD && den > -PROXIMITY_THRESHOLD)
        {
            return 0;
        }

        double s = nom/den;
        double temp = 1.0-s;
        //check if crossing point is within threshold distance from p1
        if ( nom < PROXIMITY_THRESHOLD && nom > -PROXIMITY_THRESHOLD )
        {
            p1 = p1+n*nom;
            p = p1;
            return 2;
        }
        //check if crossing point is within threshold distance from p2
        else if ( nom2 < PROXIMITY_THRESHOLD && nom2 > -PROXIMITY_THRESHOLD )
        {
            p2 = p2+n*nom2;
            p = p2;
            return 3;
        }
        //check if crossing point is within the segment
        else if (s>0 && s<1.0)
        {
            p = p1 + (s*u);
            return 1;
        }

        return 0;
    }
};
//-----------------------------------------------------------------------------
#endif
