#ifndef _LA_UTILITY_H_
#define _LA_UTILITY_H_

#include <iostream>
#include <matrix/Matrix3.h>
#include <matrix/DenseVector.h>
#include <matrix/DenseMatrix.h>
//-----------------------------------------------------------------------------
// assignment operator +=
inline Matrix3s& operator += (Matrix3s& r, const DenseVector& v)
{
  r.xx() += v(0) ; r.yy() += v(1); r.zz() += v(2);
  r.xy() += v(3) ; r.yz() += v(4); r.xz() += v(5);
  return r;
}
//-----------------------------------------------------------------------------
// Set DenseVector to the tensor from Matrix3s representation
inline DenseVector&  set_tensor( DenseVector& v, const Matrix3s& r )
{
  v(0) = r.xx(); v(1) = r.yy(); v(2) = r.zz();
  v(3) = r.xy(); v(4) = r.yz(); v(5) = r.xz();
  return v;
}
//-----------------------------------------------------------------------------
// adds to the DenseVector Voight vector in Matrix3s representation multiplied by the number
//v += d * r
inline void add_prod( double d, const Matrix3s& r, DenseVector& v)
{
  v(0) += d*r.xx();
  v(1) += d*r.yy();
  v(2) += d*r.zz();
  v(3) += d*r.xy();
  v(4) += d*r.yz();
  v(5) += d*r.xz();
}
//-----------------------------------------------------------------------------
//set dense matrix to the double array. No checks of matching dimensions done
template<int N>
void set_matrix(DenseMatrix &M, double m[][N])
{
    size_t I1 = M.size(1);
    size_t I2 = M.size(2);
    for(int i = 0; i < I1; ++i)
      for(int j = 0; j < I2; ++j)
        M.set(i,j,m[i][j]);
}
//-----------------------------------------------------------------------------
inline Vector3 operator + (Vector3 const& v3d, Vector const& vec)
{
    return Vector3(v3d.x + vec.x(), v3d.y + vec.y(), v3d.z + vec.z());
}
//-----------------------------------------------------------------------------
inline Vector3& operator += (Vector3 & v3d, Vector const& vec)
{
    v3d.x += vec.x();
    v3d.y += vec.y();
    v3d.z += vec.z();
    return v3d;
}
//-----------------------------------------------------------------------------
#endif
