// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#include <string.h>
#include <stdio.h>
#include <stdarg.h>

#include "logger.h"
#include "terminal_logger.h"
#include "utils/error_macros.h"
#include "utils/constants.h"

//-----------------------------------------------------------------------------
Logger::Logger()
{
    // Will be initialised when first needed
    log = 0;

    buffer = new char[LINE_LENGTH];
    location  = new char[LINE_LENGTH];

    state = true;
}

//-----------------------------------------------------------------------------
Logger::~Logger()
{
    if ( log )
        delete log;
    log = 0;

    if ( buffer )
        delete [] buffer;
    buffer = 0;

    if ( location )
        delete [] location;
    location = 0;
}

//-----------------------------------------------------------------------------
void Logger::begin()
{
    init();
    log->begin();
}

//-----------------------------------------------------------------------------
void Logger::end()
{
    init();
    log->end();
}

//-----------------------------------------------------------------------------
void Logger::info ( const char* msg )
{
    if ( !state )
        return;

    init();
    log->info ( msg );
}

//-----------------------------------------------------------------------------
void Logger::info ( const char* format, va_list aptr )
{
    if ( !state )
        return;

    init();

    vsprintf ( buffer, format, aptr );
    log->info ( buffer );
}

//-----------------------------------------------------------------------------
void Logger::debug ( const char* file, unsigned long line,
                     const char* function, const char* format, ... )
{
    init();

    sprintf ( location, "%s:%d: %s()", file, ( int ) line, function );

    va_list aptr;
    va_start ( aptr, format );

    vsprintf ( buffer, format, aptr );
    log->debug ( buffer, location );

    va_end ( aptr );
}

//-----------------------------------------------------------------------------
void Logger::warning ( const char* file, unsigned long line,
                       const char* function, const char* format, ... )
{
    init();

    sprintf ( location, "%s:%d: %s()", file, ( int ) line, function );

    va_list aptr;
    va_start ( aptr, format );

    vsprintf ( buffer, format, aptr );
    log->warning ( buffer, location );

    va_end ( aptr );
}
//-----------------------------------------------------------------------------
void Logger::error ( const char* file, unsigned long line,
                     const char* function, const char* format, ... )
{
    init();

    sprintf ( location, "%s:%d: %s()", file, ( int ) line, function );

    va_list aptr;
    va_start ( aptr, format );

    vsprintf ( buffer, format, aptr );
    log->error ( buffer, location );

    va_end ( aptr );
}

//-----------------------------------------------------------------------------
void Logger::cassert ( const char* file, unsigned long line,
                       const char* function, const char* format, ... )
{
    init();

    sprintf ( location, "%s:%d: %s()", file, ( int ) line, function );

    va_list aptr;
    va_start ( aptr, format );

    vsprintf ( buffer, format, aptr );
    log->cassert ( buffer, location );

    va_end ( aptr );
}

//-----------------------------------------------------------------------------
void Logger::init ( const char* destination )
{
    // Delete old logger
    if ( log )
        delete log;

    // Choose output destination
    if ( strcmp ( destination, "plain text" ) == 0 )
    {
        log = new Terminal_logger();
        return;
    }
    else
    {
        log = new Terminal_logger();
        cell_warning1 ( "Unknown output destination \"%s\", using plain text.", destination );
    }
}

//-----------------------------------------------------------------------------
void Logger::init()
{
    if ( log )
        return;

    // Default is plain text
    init ( "plain text" );
}
