// Copyright (C) 2008 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#ifndef _CELLS_PARAM_INIT_H_
#define _CELLS_PARAM_INIT_H_

#include "utils/globals.h"
#include "utils/string_utils.h"
#include "utils/error_macros.h"
#include "parameter/ParameterValue.h"
#include "parameter/ParameterList.h"

#include <sys/stat.h>
#include <dirent.h>
namespace po
{
enum Type { type_real, type_int, type_string, type_none };
}
//-----------------------------------------------------------------------------
void inline parse_options ( int ac, char* av[] )
{
    std::map<std::string, po::Type> allowed_opt;
    allowed_opt.insert ( std::make_pair ( "help", po::type_none ) );
    allowed_opt.insert ( std::make_pair ( "o", po::type_string ) );
    allowed_opt.insert ( std::make_pair ( "f", po::type_string ) );


    ParameterList par_list;

    for ( int i = 1 ; i < ac ; ++i )
    {

        std::string opt ( av[i] ) ;

        int pos = opt.find_first_not_of ( '-' );
        if ( pos > 2 || pos == 0 )
        {
            cell_warning ( ( "invalid input option format: " + opt ).c_str() );
            continue;
        }

        opt = opt.substr ( pos, opt.length() );

        std::map<std::string, po::Type>::iterator iter = allowed_opt.find ( opt );
        if ( iter == allowed_opt.end() )
        {
            cell_warning ( ( "unknown option: " + opt ).c_str() );
            continue;
        }

        po::Type val_type = iter->second;

        if ( val_type == po::type_none )
        {
            par_list.add ( opt, Parameter ( false ) );
        }
        else
        {
            ++i;
            if ( val_type == po::type_real )
            {
                double d;
                from_string ( d, av[i] );
                par_list.add ( opt, Parameter ( d ) );
            }
            else if ( val_type == po::type_int )
            {
                int d;
                from_string ( d, av[i] );
                par_list.add ( opt, Parameter ( d ) );
            }
            else if ( val_type == po::type_string )
                par_list.add ( opt, Parameter ( av[i] ) );
        }
    }
    //process supplied command line options
    if ( par_list.defined ( "help" ) )
    {
        std::cout<<"usage help to come...\n";
        exit ( 0 );
    }

    if ( par_list.defined ( "o" ) )
    {
        globals.outDir = ( std::string ) par_list.get ( "o" ) + "/";
	}
	else
	{
		globals.outDir = "output/";
	}
    if ( opendir ( globals.outDir.c_str() ) != NULL )
        std::cout<< "directory: " << globals.outDir << " already exists.\n";
    else
    {
#if defined(_WIN32)
		int stat = _mkdir ( globals.outDir.c_str());
#else 
		int stat = mkdir ( globals.outDir.c_str(),0744 ); 
#endif
        if ( stat )
            cell_error ( "Couldn't create output directory." );
    }

    if ( par_list.defined ( "f" ) )
    {
        globals.input_file = ( std::string ) par_list.get ( "f" );
    }
}
//-----------------------------------------------------------------------------
#endif
