// Copyright (C) 2007 Pawel Krupinski.
// Licensed under the GNU GPL Version 2.
#include "utils/utility.h"
#include "matrix/DenseVector.h"
#include "mesh/mesh.h"
#include <set>

//Points equal if coordinates fuzzy equal
bool fuzzy_equal(const Point &pA, const Point &pB)
{
	for(int i = 0; i <3; ++i)
	{
		bool status = fuzzy_equal( pA.cartesian(i), pB.cartesian(i) );
		if ( !status )
			return false;
	}
	return true;
}
//-----------------------------------------------------------------------------
bool Has_on::operator()( const Plane &plane, const Point& p )
{
	double eqn = plane.a() * p.x() + plane.b() * p.y() + plane.c() * p.z() + plane.d();
	return eqn<0 ? eqn>-EPS : eqn<EPS ;
}
//-----------------------------------------------------------------------------
Point centroid( Mesh const& mesh, DenseVector const& u )
{
    typedef typename Mesh::Node_const_iterator NCI;
    Vector v= CGAL::NULL_VECTOR;
    size_t nb_points = 0;
    int offset = mesh.size_of_nodes();
    for ( NCI iter = mesh.nodes_begin(); iter != mesh.nodes_end(); ++iter )
    {
        int id = iter->id();
        Point const &p = iter->point();
        Point d(u(id)+p.x(), u(id+offset)+p.y(), u(id+offset+offset)+p.z());
        v = v + ( d - CGAL::ORIGIN );
        ++nb_points;
    }
    return CGAL::ORIGIN + v / nb_points;
}
//-----------------------------------------------------------------------------
